#encoding: utf-8
module Sketchup::CoolPipe
	def self.cp_iscpcomponent?(component)                 #Определяет - является ли component компонентом CoolPipe
		rez = false
		if component!=nil
			rez = true if component.get_attribute("CoolPipeComponent","Тип")!=nil #объект CoolPipe всегда имеет свой "Тип"
		end
		rez
	end
	def self.cp_getattributes(component)                  #Получает список аттрибутов компонента CoolPipe
		attributes = {}
		if component!=nil
			atr = component.get_attribute "CoolPipeComponent", "Тип"
			attributes[:Тип]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "Dнар"
			attributes[:Dнар]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "Ду"
			attributes[:Ду]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "стенка"
			attributes[:стенка]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "Теплоизоляция"
			attributes[:Теплоизоляция]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "ГОСТ"
			attributes[:ГОСТ]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "ЕдИзм"
			attributes[:ЕдИзм]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "Имя"
			attributes[:Имя]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "УголОтвода"
			attributes[:УголОтвода]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "РадиусИзгиба"
			attributes[:РадиусИзгиба]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "D1"
			attributes[:D1]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "D2"
			attributes[:D2]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "Длина"
			attributes[:Длина]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "масса"
			attributes[:масса]=atr if atr!=nil
			atr = component.get_attribute "CoolPipeComponent", "Вариант"
			attributes[:Вариант]=atr if atr!=nil
			atr = component.material
			attributes[:Материал]=atr if atr!=nil
			atr = component.layer
			attributes[:Слой]=atr if atr!=nil
		end
		attributes
	end
	def self.cp_isconector?(face)                         #возвращает TRUE если Face - является коннектором CoolPipe
		rez = false
		if face.class==Sketchup::Face
			rez = true if face.get_attribute("CoolPipeComponent","Тип")=="Коннектор" #объект CoolPipe - Коннектор
		end
		rez
	end
	def self.cp_get_connectors_arr(component)             #РЕКУРСИВНАЯ ФУНКЦИЯ Получает все коннекторы объекта, если их нет - то возвращает nil
		connectors = []
		component_class = component.class.to_s
		case component_class
			when "Sketchup::Group"
				component.entities.each{|entity|;connectors = connectors + cp_get_connectors_arr(entity)}
			when "Sketchup::ComponentInstance"
				component.definition.entities.each{|entity|;connectors = connectors + cp_get_connectors_arr(entity)}
			else
				attribute = component.get_attribute("CoolPipeComponent","Коннектор")
				connectors << component if attribute!=nil
		end
		connectors.compact
		connectors
	end
	def self.cp_get_real_center_connectors(component)     #возвращает массив реальных центральных точек коннекторов из component
		connectors = cp_get_connectors_arr(component)
		rez = []
		if connectors!=nil
			connectors.each {|connector|
			point = Sketchup::CoolPipe::cp_set_component_transforms(component,connector.position) if connector!=nil
			rez << point if point!=nil
			}
		end
		rez
	end
	def self.cp_set_component_transforms(component,param) #применение трансформаций для param от component (например получение реальных координат центра коннектора)
		if cp_iscpcomponent?(component)
			tr = component.transformation #трансформация компонента для получения абсолютных координат
			param = param.transform! tr
		end
		param
	end
	def self.set_settings_coolpipe
		file_settings = File.join(Sketchup.find_support_file("Plugins/CoolPipe/ini/CoolPipe.ini"))
		File.open(file_settings).each do |line|
			line = line.chomp
			if (line!="") && (line!=" ") && (line!=nil)
				settings = line.split("|")
				$cp_segments      = settings[0].to_i #Количество сегментов
				$cp_elbowK        = settings[1].to_f #Коэффициент построения радиуса отвода от его диаметра
				$cp_elbowPlotnost = settings[2].to_f #Плотность материала для расчета массы отвода [г/см³]
				if (settings[3]=="checked");$cp_vnGeom = true #Строить внутреннюю геометрию
				else;$cp_vnGeom = false;end
			end
		end
	end
	class ToolbarCoolPipe                                 #Класс для создания Тулбара Coolpipe
		def initialize #Создает тулбар CoolPipe
			icon_path = get_flag_icon_path
			coolpipe_toolbar = UI::Toolbar.new "CoolPipe v.1.3(2015) Beta 3"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,'Труба',   "button_icons/pipe.jpg",    $coolpipe_langDriver['Труба'])    #Кнопка "Труба"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,'Отвод',   "button_icons/elbows.png",  $coolpipe_langDriver['Отвод'])    #Кнопка "Отвод"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,'Переход', "button_icons/reducers.png",$coolpipe_langDriver['Переход'])  #Кнопка "Переход"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,'Тройник', "button_icons/tees.jpg",    $coolpipe_langDriver['Тройник'])  #Кнопка "Тройник"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,'Заглушка',"button_icons/caps.jpg",    $coolpipe_langDriver['Заглушка']) #Кнопка "Заглушка"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,'Фланец',  "button_icons/flange.jpg",  $coolpipe_langDriver['Фланец'])   #Кнопка "Фланец"
			coolpipe_toolbar = coolpipe_toolbar.add_separator
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,'+Элемент',"button_icons/plus.png",    $coolpipe_langDriver['Превратить в элемент CoolPipe']) #Кнопка "Добавить элемент CoolPipe"
			coolpipe_toolbar = coolpipe_toolbar.add_separator
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,"Редактирование элемента","button_icons/element_edit.png",$coolpipe_langDriver['Редактирование элемента']) #Кнопка "Редактирование элемента"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,"Копирование свойств","button_icons/copy_options.png",$coolpipe_langDriver['Копирование свойств'])         #Кнопка "Копирование свойств"
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,"Спецификация","button_icons/specification.png",$coolpipe_langDriver['Составить спецификацию CoolPipe'])   #Кнопка "составить спецификацию"
			coolpipe_toolbar = delSUSANbutton(coolpipe_toolbar)			
			coolpipe_toolbar = cp_add_toolbar_button(coolpipe_toolbar,"Языковые настройки",icon_path,$coolpipe_langDriver['Языковые настройки CoolPipe']) #Кнопка "Настройки"
			coolpipe_toolbar.show
			Sketchup::CoolPipe::set_settings_coolpipe #Загрузка и установка настроек плагина			
		end
		def cp_add_toolbar_button(toolbar,command,icon,text)
			#puts text.to_s
			button = UI::Command.new(command) {
				case command
					when "Труба"
						$coolpipe_dialogs.cp_selectpipe_dialog            #Диалог выбора трубопроводов
					when "Отвод"
						Sketchup.active_model.select_tool ToolDrawElbow.new #Активизация инструмента черчения отвода
					when "Переход"
						$coolpipe_dialogs.cp_selectreducer_dialog         #Диалог выбора переходника
					when "Тройник"
						$coolpipe_dialogs.cp_selecttee_dialog             #Диалог выбора тройника
					when "Заглушка"
						Sketchup.active_model.select_tool ToolDrawCap.new #Активизация инструмента черчения заглушки
					when "Фланец"
						$coolpipe_dialogs.cp_selectflange_dialog          #Диалог выбора фланца
					when "+Элемент"
						$coolpipe_dialogs.cp_add_coolpipe_element_dialog  #Диалог превращения выделенной группы в объект Coolpipe
					when "Редактирование элемента"
						cp_edit_coolpipe_elements
					when "Копирование свойств"
						Sketchup.active_model.select_tool ToolCopyOptions.new #Активизация инструмента "копирование свойств"
					when "Спецификация"
						$coolpipe_dialogs.generate_CoolPipe_specification #Диалог спецификации
					when "Языковые настройки"
						$coolpipe_langDriver.cp_select_and_set_lang       #Диалог выбора языковой локализации проекта
						$coolpipe_langDriver = CP_LanguageHandler.new('coolpipe_lang_driver.strings') #Перезапуск языковых параметров
					when "Исправить геометрию"
					    checkAndRedrawImportGeometry
				end
				}
			button.small_icon = icon
			button.large_icon = icon
			button.tooltip = text
			button.status_bar_text = text
			button.menu_text = text
			toolbar = toolbar.add_item button
			toolbar
		end
		def cp_edit_coolpipe_elements #Редактирование элементов CoolPipe
			selection = Sketchup.active_model.selection
			count = selection.count
			is_component = Sketchup::CoolPipe::cp_iscpcomponent?(selection[0])
			@@cp_edit_pipe_enable  = true
			if (count==1)&&(is_component)
					attributes = Sketchup::CoolPipe::cp_getattributes(selection[0])
					name = attributes[:Имя] 
					type = attributes[:Тип] 
					if (type!=nil)
						view = Sketchup.active_model.active_view
						case type #Запуск модулей редактирования элементов
							when "Труба"
								$cp_change_pipe_tool = ToolEditPipe.new(name,selection[0],selection)
								Sketchup.active_model.select_tool $cp_change_pipe_tool
							when "Отвод"
								UI.messagebox($coolpipe_langDriver['Редактирование отвода не предусмотрено'])
								#Sketchup.active_model.select_tool ToolEditElbow.new(name,selection[0],selection)
								#Sketchup.active_model.select_tool CP_EditElbowTool.new(name,selection[0],selection)
							when "Переход"
								$cp_change_reducer_tool = ToolEditReducer.new(name,selection[0],selection)
								Sketchup.active_model.select_tool $cp_change_reducer_tool
							when "Тройник"
								$cp_change_tee_tool = ToolEditTee.new(name,selection[0],selection)
								Sketchup.active_model.select_tool $cp_change_tee_tool
							when "Заглушка"
								UI.messagebox($coolpipe_langDriver['Редактирование заглушки не предусмотрено'])
							when "Фланец"
								UI.messagebox($coolpipe_langDriver['Редактирование фланца не предусмотрено'])
						end
					view.invalidate
				end
			else
				UI.messagebox($coolpipe_langDriver["Необходимо выбрать компонент CoolPipe для редактирования"])
			end
		end
		def get_flag_icon_path
			locale = $coolpipe_langDriver.get_active_lang 
			case locale
				when "ru"
					icon_path = "button_icons/lang/Russia.png"
				when "en"
					icon_path = "button_icons/lang/United-Kingdom.png"
				when "fr"
					icon_path = "button_icons/lang/France.png"
				when "it"
					icon_path = "button_icons/lang/Italy.png"
				when "de"
					icon_path = "button_icons/lang/Germany.png"
				when "sp"
					icon_path = "button_icons/lang/Spain.png"
				when "ch"
					icon_path = "button_icons/lang/China.png"
				else
					icon_path = "button_icons/settings.png"
			end
			icon_path
		end
		def delSUSANbutton(toolbar)
			delSusanBut = UI::Command.new("Delete Susan, Derrick and purge unused") {
				entities = Sketchup.active_model.entities
				Sketchup.active_model.definitions.purge_unused
				entities.each{|entity|
				if (entity.valid?)and(entity.class==Sketchup::ComponentInstance)
					if ((entity.definition.name.to_s=="Susan")and(entity.definition.description.to_s=="Susan is a member of the SketchUp development team.")) or
					   ((entity.definition.name.to_s=="Derrick")and(entity.definition.description.to_s=="Derrick was a great friend and supporter of SketchUp. He loved spending time with his wife Sharon, his sons Josh, Nick and Will, and his loyal dog Huck.")) or
					   ((entity.definition.name.to_s=="Sophie")and(entity.definition.description.to_s=="Sophie is a member of the SketchUp Pro business development team. She dreams of traveling the world, eating good food and taking pictures of beautiful places.")) or
					   ((entity.definition.name.to_s=="Steve")and(entity.definition.description.to_s=="Paul Stevenson Oles, FAIA, is an architect and co-founder of the American Society of Architectural Illustrators. Steve takes pleasure in long walks, working out, designing all sorts of things, and riding the roads of New Mexico on his beloved '08 Triumph Bonneville."))
						entity.erase!
					end
				end;}
			}
			delSusanBut.small_icon = "button_icons/del_Susan.jpg"
			delSusanBut.large_icon = "button_icons/del_Susan.jpg"
			text = $coolpipe_langDriver["Удалить объекты [Сюзан,Детрик,Софи,Стив] и очистка проекта от неиспользуемых объектов"]
			delSusanBut.tooltip = text
			delSusanBut.status_bar_text = text
			delSusanBut.menu_text = text
			toolbar = toolbar.add_item delSusanBut
			toolbar
		end #def delSUSANbutton(toolbar)
	end #class PluginCoolPipe
	class CoolPipeDialogs                                 #В этом классе сосредоточены все диалоги
		###################
		def cp_refreshdialog(dialog,dialogname)          # Обновляет содержимое диалога (переоткрывая его)
			if (dialog!=nil) && (dialog.class==UI::WebDialog)
			   dialog.close
			end
			case dialogname
				when "selectpipe_dialog"
					cp_selectpipe_dialog
				when "selectreducer_dialog"
					cp_selectreducer_dialog
				when "selecttee_dialog"
					cp_selecttee_dialog
				when "selectflange_dialog"
					cp_selectflange_dialog
			end
		end
		def cp_add_coolpipe_element_dialog               # Диалог добавления атрибутов CoolPipe произвольной группе элементов
			model = Sketchup.active_model
			selection = model.selection
			if selection.count>1
				# Сюда вставить блок по предложению об обединений нескольких объектов в группу!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			end
			@cppr_selected_obj = selection[0]
			if (@cppr_selected_obj!=nil) && (selection.count==1)
				selclass = @cppr_selected_obj.class
				#puts $coolpipe_langDriver["Класс выделенного объекта = "]+selclass.to_s
				is_cp_obj = Sketchup::CoolPipe::cp_iscpcomponent?(@cppr_selected_obj)
				case is_cp_obj
					when true
						puts $coolpipe_langDriver["Объект является компонентом CoolPipe"]
						UI.messagebox($coolpipe_langDriver["Объект является компонентом CoolPipe"],MB_OK)
						attributes = Sketchup::CoolPipe::cp_getattributes(@cppr_selected_obj)
					when false
						puts $coolpipe_langDriver["Объект не является компонентом CoolPipe"]
				end
				#if is_cp_obj==false #если объект не является компонентом CoolPipe то вызываем диалоговое окно для установки атрибутов CoolPipe
					dlg = UI::WebDialog.new($coolpipe_langDriver["Диалог назначения атрибутов CoolPipe"], true,
					$coolpipe_langDriver["Диалог назначения атрибутов CoolPipe"], 550, 450, 100, 100, true);#width,height,left,top
					@cppr_main_dialog = dlg #глобальная переменная доступа к активному диалогу
					path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_coolpipe_element.html"))
					dlg.set_file(path)
					dlg.set_background_color(dlg.get_default_dialog_color)
					dlg.add_action_callback("ValueChanged") {|dialog, params|
						params = dlg.get_element_value("Alt_CallBack") if params==nil
						arr=params.split("|")
						case arr[0]
							when "load_succesfull"
								if is_cp_obj==true
									dlg.execute_script("document.getElementById('Type_').value=\"#{attributes[:Тип]}\"")
									dlg.execute_script("document.getElementById('type_sel').value=\"#{attributes[:Тип]}\"") #Инициация выпадающего списка
									dlg.execute_script("document.getElementById('Ду').value=\"#{attributes[:Ду]}\"")
									dlg.execute_script("document.getElementById('Dнар').value=\"#{attributes[:Dнар]}\"")
									dlg.execute_script("document.getElementById('стенка').value=\"#{attributes[:стенка]}\"")
									dlg.execute_script("document.getElementById('Имя').value=\"#{attributes[:Имя]}\"")
									dlg.execute_script("document.getElementById('ЕдИзм').value=\"#{attributes[:ЕдИзм]}\"")
									dlg.execute_script("document.getElementById('масса').value=\"#{attributes[:масса]}\"")
								end
								#Устанавливаем текущие языковые параметры
								dlg.execute_script("document.getElementById('text_type').innerHTML=\"#{$coolpipe_langDriver["Тип"]}\"")
								dlg.execute_script("document.getElementById('text_diametrUslovniy').innerHTML=\"#{$coolpipe_langDriver["Диаметр условный"]}\"")
								dlg.execute_script("document.getElementById('text_diametrNaruzniy').innerHTML=\"#{$coolpipe_langDriver["Диаметр наружний"]}\"")
								dlg.execute_script("document.getElementById('text_tolsinaStenki').innerHTML=\"#{$coolpipe_langDriver["Толщина стенки"]}\"")
								dlg.execute_script("document.getElementById('text_name').innerHTML=\"#{$coolpipe_langDriver["Наименование"]}\"")
								dlg.execute_script("document.getElementById('text_edIzm').innerHTML=\"#{$coolpipe_langDriver["Единица измерения"]}\"")
								dlg.execute_script("document.getElementById('text_massaEd').innerHTML=\"#{$coolpipe_langDriver["Масса единицы"]}\"")
								dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
								dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
								dlg.execute_script("document.getElementById('ЕдИзм').value=\"#{$coolpipe_langDriver["шт"]}\"")								
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Отвод"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Переход"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Тройник"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Заглушка"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Фланец"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Основное оборудование"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Арматура"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
								dlg.execute_script("a=\"#{$coolpipe_langDriver["Другое"]}\";selectbox.options[selectbox.options.length] = new Option(a,a);")
							when "cancel"
								dlg.close
							when "save"
								attributes = {}
								attributes[:Тип]    = dlg.get_element_value("Type_") 
								attributes[:Ду]     = dlg.get_element_value("Ду") 
								attributes[:Dнар]   = dlg.get_element_value("Dнар")
								attributes[:стенка] = dlg.get_element_value("стенка")
								attributes[:Имя]    = dlg.get_element_value("Имя")
								attributes[:ЕдИзм]  = dlg.get_element_value("ЕдИзм")
								attributes[:масса]  = dlg.get_element_value("масса")
								puts "--------------------------------------------"
								puts $coolpipe_langDriver["Получены следующие аттрибуты:"]
								puts "Тип    => "+attributes[:Тип].to_s
								puts "Ду     => "+attributes[:Ду].to_s
								puts "Dнар   => "+attributes[:Dнар].to_s
								puts "стенка => "+attributes[:стенка].to_s
								puts "Имя    => "+attributes[:Имя].to_s
								puts "ЕдИзм  => "+attributes[:ЕдИзм].to_s
								puts "масса  => "+attributes[:масса].to_s
								puts "--------------------------------------------"
								attributes.each_pair do |key, value|
									@cppr_selected_obj.set_attribute "CoolPipeComponent",key.to_s,value.to_s
								end
								dlg.close
						end
						}
					dlg.max_height = 450
					dlg.max_width  = 550
					dlg.min_height = 300
					dlg.min_width  = 300
					dlg.set_on_close{@cppr_main_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
					dlg.show_modal
				#end #if @cp_change_list_pipes_du_dialog==nil
			else
				UI.messagebox($coolpipe_langDriver["Необходимо выбрать группу или компонент SketchUp, который необходимо присоединить к CoolPipe"],MB_OK)
				puts $coolpipe_langDriver["Необходимо выбрать один компонент"]
			end
		end #cp_add_coolpipe_element_dialog
		##################################################################################################
		#------- Трубопроводы: --------
		##################################################################################################
		def cp_selectpipe_dialog(change_diametr = false)                #Диалог выбора трубопроводов
			if @cp_activ_selectpipe_dialog==nil
					@dialog_ini = File.join(Sketchup.find_support_file("Plugins/CoolPipe/ini/pipe_dialog.ini"))
					@cp_pipe_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/pipe_base.csv")))
					@cp_layers_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/layer_base.csv"))) 
					dlg = UI::WebDialog.new($coolpipe_langDriver["Выбор трубопровода"], true,$coolpipe_langDriver["Выбор трубопровода"], 475, 450, 100, 100, true);#width,height,left,top
					@cp_activ_selectpipe_dialog = dlg #глобальная переменная доступа к активному диалогу
					path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/select_pipe_dialog.html"))
					dlg.set_file(path)
					dlg.set_background_color(dlg.get_default_dialog_color)
					dlg.add_action_callback("ValueChanged") {|dialog, params|
					if params==nil
						params = dlg.get_element_value("Alt_CallBack") 
					end
				arr = params.split("|")
				check_commands_selectpipe_dialog(arr,dlg,change_diametr)  #Проверка и обработка комманд поступающих от диалогового окна
				} # end dlg.add_action_callback
				dlg.max_height = 450
				dlg.max_width  = 600
				dlg.min_height = 450
				dlg.min_width  = 500
				dlg.set_on_close{
				#сохранение последнего состояния окна
				ss_base = dlg.get_element_value("Base_Type") 
				ss_doc = dlg.get_element_value("Document_Name") 
				ss_du = dlg.get_element_value("Du_Select") 
				ss_needlayer = dlg.get_element_value("checked_layers") 
				ss_needmaterial = dlg.get_element_value("checked_materials") 
				ss_layer = dlg.get_element_value("idlayerselect") 
				text = ss_base+"|"+ss_doc+"|"+ss_du+"|"+ss_needlayer+"|"+ss_needmaterial+"|"+ss_layer
				File.open(@dialog_ini, "w") do |file|
					file.puts text 
				end
				@cp_activ_selectpipe_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
				dlg.show
			end #if @cp_activ_selectpipe_dialog==nil
		end #def cp_selectpipe_dialog
		def check_commands_selectpipe_dialog(arr,dlg,change_diametr)    #Проверка комманд от диалога добавления наименований документов
			case arr[0]
			###########
				#если окно загруженно заполняем его    
				when "load_succesfull"     
					typedocs = @cp_pipe_database.get_row_uniq("Тип_документа")
					typedocs.each do |typedoc|
						typedoc = typedoc.chomp
						js_command = "document.getElementById('Base_Type').options[document.getElementById('Base_Type').options.length]=new Option('#{typedoc}','#{typedoc}');"
						dlg.execute_script(js_command) #запоняем список базы данных
					end
					layers = @cp_layers_database.get_layers_name
					layers.each {|layer|
						js_command = "document.getElementById('idlayerselect').options[document.getElementById('idlayerselect').options.length]=new Option('#{layer}','#{layer}');"
						dlg.execute_script(js_command) #запоняем список слоев
					}
					#восстановление предыдущего состояния
					File.open(@dialog_ini).each do |line|
						line = line.chomp
						if (line!="") && (line!=" ") && (line!=nil)
							restore = line.split("|")
							dlg.execute_script("document.getElementById('Base_Type').value=\"#{restore[0]}\"")
							dlg.execute_script("fsetactivebase()")
							dlg.execute_script("document.getElementById('Document_Name').value=\"#{restore[1]}\"")
							dlg.execute_script("fsetactivedocument()")
							dlg.execute_script("document.getElementById('Du_Select').value=\"#{restore[2]}\"")
							dlg.execute_script("fsetactivedu()")
							dlg.execute_script("document.getElementById('idputlayers').checked=#{restore[3]}")
							dlg.execute_script("fcheckassign(document.getElementById('idputlayers'))")
							dlg.execute_script("document.getElementById('idputmaterial').checked=#{restore[4]}")
							dlg.execute_script("fcheckassign(document.getElementById('idputmaterial'))")
							dlg.execute_script("document.getElementById('idlayerselect').value=\"#{restore[5]}\"")
						end
						#Устанавливаем текущие языковые параметры
						dlg.execute_script("document.getElementById('text_database').innerHTML=\"#{$coolpipe_langDriver["База данных:"]}\"")
						dlg.execute_script("document.getElementById('text_document').innerHTML=\"#{$coolpipe_langDriver["Документ:"]}\"")
						dlg.execute_script("document.getElementById('text_nameofdocument').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
						dlg.execute_script("document.getElementById('text_uslovndiametr').innerHTML=\"#{$coolpipe_langDriver["Условный диаметр:"]}\"")
						dlg.execute_script("document.getElementById('text_nametube').innerHTML=\"#{$coolpipe_langDriver["Наименование:"]}\"")
						dlg.execute_script("document.getElementById('text_needLayers').innerHTML=\"#{$coolpipe_langDriver["Использовать слои"]}\"")
						dlg.execute_script("document.getElementById('text_needmaterials').innerHTML=\"#{$coolpipe_langDriver["Использовать материалы"]}\"")
						dlg.execute_script("document.getElementById('text_draw').value=\"#{$coolpipe_langDriver["Чертить"]}\"")
						dlg.execute_script("document.getElementById('text_cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
						dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
					end
				###########
				#выбран активный тип документов (например ГОСТ)
				when "set_activ_base" 
					name_filter = arr[1]
					name_filter = encode_to_utf8(name_filter)
					namedocs = @cp_pipe_database.get_row_uniq_filter1("Тип_документа",name_filter,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
							if (namedoc!="") && (namedoc!=nil)
								js_command = "document.getElementById('Document_Name').options[document.getElementById('Document_Name').options.length]=new Option('#{namedoc}','#{namedoc}');"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
				###########
				#выбран активный документ из базы
				when "set_activ_document" 
					typedoc = arr[1]
					name_filter = arr[2]
					typedoc     = encode_to_utf8(typedoc)
					name_filter = encode_to_utf8(name_filter)
					diametrs = @cp_pipe_database.get_row_uniq_filter2("Тип_документа",typedoc,"Документ",name_filter,"Ду")
					descriptdoc = @cp_pipe_database.get_descript_doc("Документ",name_filter,"Описание_документа")
					js_command = "document.getElementById('Name_DOC').innerHTML=\"#{descriptdoc}\";"
					dlg.execute_script(js_command) #показываем описание документа
					if diametrs.length>0
						diametrs.each do |diametr|
							js_command = "document.getElementById('Du_Select').options[document.getElementById('Du_Select').options.length]=new Option('#{diametr}','#{diametr}');"
							dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
				###########
				when "set_activ_du" #выбран условный диаметр из базы
					typedoc = arr[1]
					docname = arr[2]
					du = arr[3]
					typedoc = encode_to_utf8(typedoc)
					docname = encode_to_utf8(docname)
					du      = encode_to_utf8(du)
					nametube = @cp_pipe_database.get_descript_element("Документ",docname,"Ду",du,"Наименование_трубопровода")
					js_command = "document.getElementById('Name_Pipe').innerHTML=\"#{nametube}\";"
					dlg.execute_script(js_command) #пишем наименование трубы
				###########
				when "change_list_base"
					cp_change_list_pipebase_dialog
				when "change_list_document"
					base = arr[1]
					base = encode_to_utf8(base)
					cp_change_list_pipe_documents_dialog(base)
				when "change_list_pipes_du"
					base = arr[1]
					doc = arr[2]
					base = encode_to_utf8(base)
					doc  = encode_to_utf8(doc)
					cp_change_list_pipes_du(base,doc)
				when "change_list_layers"
					cp_change_list_layers_dialog
				when "cancel"
					dlg.close
				when "draw_pipe" #рисуем трубопровод
					base = dlg.get_element_value("Base_Type") 
					doc = dlg.get_element_value("Document_Name") 
					du = dlg.get_element_value("Du_Select") 
					need_layer = dlg.get_element_value("checked_layers") 
					need_material = dlg.get_element_value("checked_materials") 
					layer = dlg.get_element_value("idlayerselect") if need_layer=="true"
					layer = nil if need_layer=="false"
					material = @cp_layers_database.get_sketch_color(layer) if (need_material=="true") && (need_layer=="true")
					material = nil if need_material=="false"
					massa    = @cp_pipe_database.get_tube_params(base,doc,du,"Масса")
					dnar     = @cp_pipe_database.get_tube_params(base,doc,du,"Дн")
					stenka   = @cp_pipe_database.get_tube_params(base,doc,du,"Стенка")
					nametube = @cp_pipe_database.get_tube_params(base,doc,du,"Наименование_трубопровода")
					#nametube =$coolpipe_langDriver["Труба"]+" Ø#{dnar}x#{stenka} #{doc}"
					param={:Тип            => "Труба",       #Тип компонента: труба
							:База           => base,         #Тип базы ГОСТ, ТУ или др...
							:ГОСТ           => doc,          #Нормативный документ (из базы)
							:Ду             => du,           #Диаметр трубопровода условный
							:Имя            => nametube,     #Наименование трубы для спецификации
							:ЕдИзм          => $coolpipe_langDriver["п.м."],       #Единица измерения для спецификации
							:масса          => massa,        #Масса единицы для спецификации
							:Dнар           => dnar,         #Диаметр трубопровода наружний
							:стенка         => stenka,       #Толщина стенки трубопровода
							:Установить_слой=> need_layer,   #Флаг установки слоя
							:Установить_материал=> need_material,   #Флаг установки материала
							:Материал       => material,     #Материал трубопровода (собственный цвет из настроек слоев)-если нет то 0
							:Имя_слоя       => layer,        #имя слоя, если нет - то 0
							:Теплоизоляция  => "0",
							:Сегментов      => $cp_segments
							}
					dlg.close
					Sketchup.active_model.select_tool ToolDrawPipe.new(param) if change_diametr==false #АКТИВАЦИЯ ТУЛА ЧЕРЧЕНИЯ ТРУБОПРОВОДА
					#Sketchup.active_model.select_tool CP_DrawPipeTool.new(param) if change_diametr==false #старая АКТИВАЦИЯ ТУЛА ЧЕРЧЕНИЯ ТРУБОПРОВОДА
					if change_diametr==true #Изменение диаметра
						$cp_change_pipe_tool.change_diam(param) if $cp_change_pipe_tool!=nil
					end
			end
		end #def check_commands_selectpipe_dialog(arr,dlg,change_diametr)
		###################
		def cp_change_list_pipebase_dialog                              #Диалог редактирования списка баз по трубопроводам ГОСТ/ТУ/DIN и т.д.
			if @cp_activ_changelistpipebase_dialog==nil
			@cp_pipe_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/pipe_base.csv"))) if @cp_pipe_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список баз по трубам"], true,$coolpipe_langDriver["Список баз по трубам"], 350, 350, 100, 100, true);#width,height,left,top
			@cp_activ_changelistpipebase_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/change_list_pipebase_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			if params==nil
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_change_list_pipebase_dialog(arr,dlg)  #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 350
			dlg.max_width  = 350
			dlg.min_height = 350
			dlg.min_width  = 350
			dlg.set_on_close{@cp_activ_changelistpipebase_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_activ_changelistpipebase_dialog==nil
		end #cp_change_list_pipebase_dialog
		def check_commands_change_list_pipebase_dialog(arr,dlg)
			case arr[0]
				when "listbase_load_succesfull" #если окно загруженно заполняем его
					typedocs = @cp_pipe_database.get_row_uniq("Тип_документа")
					if typedocs.length>0
						typedocs.each do |typedoc|
							if (typedoc!="") && (typedoc!=nil)
								js_command = "addrow('#{typedoc}')"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
					#Устанавливаем текущие языковые параметры
					 dlg.execute_script("document.getElementById('text_editbases').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка баз"]}\"")
					 dlg.execute_script("document.getElementById('text_typedocuments').innerHTML=\"#{$coolpipe_langDriver["Тип документов"]}\"")
					 dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					 dlg.execute_script("document.getElementById('newbase').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					 dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					 dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					 dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistbase"
					#Редактируем список с базами
					for i in 1..(arr.length-1)
						arr[i] = encode_to_utf8(arr[i])
						@cp_pipe_database.add_newtypedoc(arr[i])
					end
					dlg.close
					cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
				when "delete"
					arr[1] = encode_to_utf8(arr[1])
					@cp_pipe_database.delete_typedoc(arr[1])
					cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
			end
		end
		#########
		def cp_change_list_pipe_documents_dialog(base)                  #Диалог редактирования списка документов в соответствующей базе
			if @cp_activ_changelistpipedocuments_dialog==nil
				@cp_pipe_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/pipe_base.csv"))) if @cp_pipe_database==nil #Загружаем базу если она не загруженна
				dlg = UI::WebDialog.new($coolpipe_langDriver["Список документов по трубам"], true,$coolpipe_langDriver["Список документов по трубам"], 550, 400, 100, 100, true);#width,height,left,top
				@cp_activ_changelistpipedocuments_dialog = dlg #глобальная переменная доступа к активному диалогу
				path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_pipe_document_dialog.html"))
				dlg.set_file(path)
				dlg.set_background_color(dlg.get_default_dialog_color)
				dlg.add_action_callback("ValueChanged") {|dialog, params|
				if params==nil
					params = dlg.get_element_value("Alt_CallBack") 
				end
				arr=params.split("|")
				check_commands_list_pipe_documents_dialog(arr,arr[0],base,dlg)  #Проверка и обработка комманд поступающих от диалогового окна
				}
				dlg.max_height = 400
				dlg.max_width  = 550
				dlg.min_height = 400
				dlg.min_width  = 550
				dlg.navigation_buttons_enabled = false
				dlg.set_on_close{@cp_activ_changelistpipedocuments_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
				dlg.show
			end #if @cp_activ_changelistpipedocuments_dialog==nil
		end #cp_change_list_pipe_documents_dialog
		def check_commands_list_pipe_documents_dialog(arr,command,base,dlg) #Проверка комманд от диалога добавления наименований документов
			case command
				when "listdocuments_load_succesfull" #если окно загруженно заполняем его
					namedocs = @cp_pipe_database.get_row_uniq_filter1("Тип_документа",base,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
							if (namedoc!="") && (namedoc!=nil)
								descriptdoc = @cp_pipe_database.get_descript_doc("Документ",namedoc,"Описание_документа")
								js_command = "addrow(\"#{namedoc}\",\"#{descriptdoc}\")"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_editdoclist').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка документов"]}\"")
					dlg.execute_script("document.getElementById('text_namedoc').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
					dlg.execute_script("document.getElementById('text_descriptdoc').innerHTML=\"#{$coolpipe_langDriver["Описание документа"]}\"")
					dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
					#@cp_activ_changelistpipedocuments_dialog = nil
				when "skp:ValueChanged@changelistdocuments"
					dlg.close
					#@cp_activ_changelistpipedocuments_dialog = nil
				when "changelistdocuments"
					#Редактируем список с базами
					@cp_activ_changelistpipedocuments_dialog.close
					if arr[1]!="0"
						for i in 1..(arr.length-1)
							arr2 = arr[i].split("=")
							@cp_pipe_database.add_newdoc(base,encode_to_utf8(arr2[0]),encode_to_utf8(arr2[1]))
						end
						cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
					end
					#@cp_activ_changelistpipedocuments_dialog = nil
				when "delete"
					@cp_pipe_database.delete_doc(base,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
			end
		end #check_commands_list_pipe_documents_dialog(command)
		#########
		def cp_change_list_pipes_du(base,doc)                           #Диалог редактирования списка диаметров в документе принадлежащему базе
			if @cp_change_list_pipes_du_dialog==nil
			@cp_pipe_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/pipe_base.csv"))) if @cp_pipe_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список трубопроводов"]+doc, true,$coolpipe_langDriver["Список трубопроводов"], 750, 400, 100, 100, true);#width,height,left,top
			@cp_change_list_pipes_du_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_pipe_element_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			if params==nil
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_change_list_pipes_du(arr,base,doc,dlg)  #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 400
			dlg.max_width  = 750
			dlg.min_height = 400
			dlg.min_width  = 750
			dlg.set_on_close{@cp_change_list_pipes_du_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_change_list_pipes_du_dialog==nil
		end #cp_change_list_pipes_du
		def check_commands_change_list_pipes_du(arr,base,doc,dlg) #Проверка и обработка комманд поступающих от диалогового окна
			case arr[0]
				when "listdu_load_succesfull" #если окно загруженно заполняем его
					namedu = @cp_pipe_database.get_row_uniq_filter2("Тип_документа",base,"Документ",doc,"Ду")
					if namedu.length>0
						namedu.each do |du|
							namedn = @cp_pipe_database.get_tube_params(base,doc,du,"Дн")
							namest = @cp_pipe_database.get_tube_params(base,doc,du,"Стенка")
							namemas = @cp_pipe_database.get_tube_params(base,doc,du,"Масса")
							namedescript = @cp_pipe_database.get_tube_params(base,doc,du,"Наименование_трубопровода")
							if (du!="") && (du!=nil) #addrow(du,dn,st,mas,name)
								js_command = "addrow(\"#{du}\",\"#{namedn}\",\"#{namest}\",\"#{namemas}\",\"#{namedescript}\")"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
							#Устанавливаем текущие языковые параметры
							dlg.execute_script("document.getElementById('text_editListpipes').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка трубопроводов"]}\"")
							dlg.execute_script("document.getElementById('text_DU').innerHTML=\"#{$coolpipe_langDriver["Диаметр условный"]}\"")
							dlg.execute_script("document.getElementById('text_DN').innerHTML=\"#{$coolpipe_langDriver["Диаметр наружний"]}\"")
							dlg.execute_script("document.getElementById('text_Stenka').innerHTML=\"#{$coolpipe_langDriver["Толщина стенки"]}\"")
							dlg.execute_script("document.getElementById('text_Massa').innerHTML=\"#{$coolpipe_langDriver["Масса"]}\"")
							dlg.execute_script("document.getElementById('text_NameForSpec').innerHTML=\"#{$coolpipe_langDriver["Наименование для спецификации"]}\"")
							dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
							dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
							dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
							dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
							dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
						end
					end
				when "cancel"
					dlg.close
				when "changelistpipeelements"
					#Редактируем список с базами
					@cp_change_list_pipes_du_dialog.close
					arr[1]=encode_to_utf8(arr[1])
					if arr[1]!="0"
						listdu = arr
						listdu = listdu.uniq
						listdu=listdu.collect{|a|a=encode_to_utf8(a)}
						@cp_pipe_database.add_tubes(base,doc,listdu)
						cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
					end
				when "delete"
					@cp_pipe_database.delete_tube(base,doc,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
			end
		end #check_commands_change_list_pipes_du(arr,base,doc,dlg)
		#########
		def cp_change_list_layers_dialog                                #Диалог редактирования списка слоев CoolPipe
			if @cp_change_list_layers_dialog ==nil
			@cp_layers_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/layer_base.csv"))) if @cp_layers_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список слоев CoolPipe"], true,$coolpipe_langDriver["Список слоев CoolPipe"], 725, 600, 100, 100, true);#width,height,left,top
			@cp_change_list_layers_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_layers_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			if params==nil
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_list_layers_dialog(arr,dlg)  #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 600
			dlg.max_width  = 725
			dlg.min_height = 600
			dlg.min_width  = 725
			dlg.set_on_close{@cp_change_list_layers_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_change_list_layers_dialog==nil
		end #cp_change_list_layers_dialog
		def check_commands_list_layers_dialog(arr,dlg)
			case arr[0]
				when "listlayers_load_succesfull" #если окно загруженно заполняем его
					layers = @cp_layers_database.get_layers_name
					colors = @cp_layers_database.get_colors_HTMLname
					i = 0
					layers.each {|layer|
					js_command = "addrow(\"#{layer}\",\"#{colors[i]}\")"
					dlg.execute_script(js_command) #запоняем список слоев
					i+=1
					}
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_namelayer').innerHTML=\"#{$coolpipe_langDriver["Имя слоя"]}\"")
					dlg.execute_script("document.getElementById('text_color').innerHTML=\"#{$coolpipe_langDriver["Цвет"]}\"")
					dlg.execute_script("document.getElementById('text_selectedcolor').innerHTML=\"#{$coolpipe_langDriver["Выбранный цвет"]}\"")
					dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					dlg.execute_script("text_editcolor=\"#{$coolpipe_langDriver["Редактирование цвета слоя"]}\"")
					dlg.execute_script("document.getElementById('newlayer').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					dlg.execute_script("document.getElementById('cancel_color').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('save_color').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistlayers"
					dlg.close
					layers = []
					for i in arr
						a = encode_to_utf8(i)
						layers << a if a!="changelistlayers"
					end
					@cp_layers_database.save_layers(layers)
					cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
				when "delete"
					@cp_layers_database.delete_layer(name)
					cp_refreshdialog(@cp_activ_selectpipe_dialog,"selectpipe_dialog")
			end
		end #check_commands_list_layers_dialog(arr,dlg)
		##################################################################################################
		#------- Переходы
		##################################################################################################
		def cp_selectreducer_dialog(change_diametr = false) #Диалог выбора переходов
			if @cp_activ_selectreducer_dialog==nil
				dialog_ini = File.join(Sketchup.find_support_file("Plugins/CoolPipe/ini/reducer_dialog.ini"))
				@cp_reducer_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/reducer_base.csv")))
				dlg = UI::WebDialog.new($coolpipe_langDriver["Выбор перехода"], true,$coolpipe_langDriver["Выбор перехода"], 475, 450, 100, 100, true);#width,height,left,top
				@cp_activ_selectreducer_dialog = dlg #глобальная переменная доступа к активному диалогу
				path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/select_reducer_dialog.html"))
				dlg.set_file(path)
				dlg.set_background_color(dlg.get_default_dialog_color)
				dlg.add_action_callback("ValueChanged") {|dialog, params|
					if params==nil
						params = dlg.get_element_value("Alt_CallBack") 
					end
					arr=params.split("|")
					check_commands_selectreducer_dialog(arr,dlg,change_diametr,dialog_ini) #Проверка и обработка комманд поступающих от диалогового окна
				}   #dlg.add_action_callback("ValueChanged") {|dialog, params|
				dlg.max_height = 450
				dlg.max_width  = 600
				dlg.min_height = 450
				dlg.min_width  = 500
				dlg.set_on_close{
				#сохранение последнего состояния окна
				ss_base = dlg.get_element_value("Base_Type") 
				ss_doc = dlg.get_element_value("Document_Name") 
				ss_du1 = dlg.get_element_value("Du1_Select") 
				ss_du2 = dlg.get_element_value("Du2_Select") 
				text = ss_base+"|"+ss_doc+"|"+ss_du1+"|"+ss_du2
				File.open(dialog_ini, "w") do |file|
					file.puts text 
				end
				@cp_activ_selectreducer_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
				dlg.show
			end #if @cp_activ_selectreducer_dialog==nil
		end
		def check_commands_selectreducer_dialog(arr,dlg,change_diametr,dialog_ini)
			case arr[0]
				when "load_succesfull"
					typedocs = @cp_reducer_database.get_row_uniq("Тип_документа")
					typedocs.each do |typedoc|
						typedoc = typedoc.chomp
						js_command = "document.getElementById('Base_Type').options[document.getElementById('Base_Type').options.length]=new Option('#{typedoc}','#{typedoc}');"
						dlg.execute_script(js_command) #запоняем список базы данных
					end
					#восстанавливаем последнее состояние диалога
					File.open(dialog_ini).each do |line|
						line = line.chomp
						if (line!="") && (line!=" ") && (line!=nil)
							restore = line.split("|")
							dlg.execute_script("document.getElementById('Base_Type').value=\"#{restore[0]}\"")
							dlg.execute_script("fsetactivebase()")
							dlg.execute_script("document.getElementById('Document_Name').value=\"#{restore[1]}\"")
							dlg.execute_script("fsetactivedocument()")
							dlg.execute_script("document.getElementById('Du1_Select').value=\"#{restore[2]}\"")
							dlg.execute_script("fsetactivedu1()")
							dlg.execute_script("document.getElementById('Du2_Select').value=\"#{restore[3]}\"")
							dlg.execute_script("fsetactivedu2()")
						end
					end
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_database').innerHTML=\"#{$coolpipe_langDriver["База данных"]}\"")
					dlg.execute_script("document.getElementById('text_document').innerHTML=\"#{$coolpipe_langDriver["Документ:"]}\"")
					dlg.execute_script("document.getElementById('text_nameofdocument').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
					dlg.execute_script("document.getElementById('text_DN1').innerHTML=\"#{$coolpipe_langDriver["Ду"]}1\"")
					dlg.execute_script("document.getElementById('text_DN2').innerHTML=\"#{$coolpipe_langDriver["Ду"]}2\"")
					dlg.execute_script("document.getElementById('text_namereducer').innerHTML=\"#{$coolpipe_langDriver["Наименование:"]}\"")
					dlg.execute_script("document.getElementById('text_draw').value=\"#{$coolpipe_langDriver["Чертить"]}\"")
					dlg.execute_script("document.getElementById('text_cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "change_list_base"
					cp_change_list_reduserbase_dialog
				when "change_list_document"
					base = arr[1]
					base = encode_to_utf8(base)
					cp_change_list_reduser_documents_dialog(base)
				when "change_list_reducers"
					base = arr[1]
					doc  = arr[2]
					base = encode_to_utf8(base)
					doc  = encode_to_utf8(doc)
					cp_change_list_redusers(base,doc)
				when "set_activ_base"
					name_filter = arr[1]
					name_filter = encode_to_utf8(name_filter)
					namedocs = @cp_reducer_database.get_row_uniq_filter1("Тип_документа",name_filter,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
							if (namedoc!="") && (namedoc!=nil)
								js_command = "document.getElementById('Document_Name').options[document.getElementById('Document_Name').options.length]=new Option('#{namedoc}','#{namedoc}');"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
				when "set_activ_document"
					typedoc = arr[1]
					name_filter = arr[2]
					typedoc     = encode_to_utf8(typedoc)
					name_filter = encode_to_utf8(name_filter)
					diametrs = @cp_reducer_database.get_row_uniq_filter2("Тип_документа",typedoc,"Документ",name_filter,"DN1")
					descriptdoc = @cp_reducer_database.get_descript_doc("Документ",name_filter,"Описание_документа")
					js_command = "document.getElementById('Name_DOC').innerHTML=\"#{descriptdoc}\";"
					dlg.execute_script(js_command) #показываем описание документа
					if diametrs.length>0
						diametrs.each do |diametr|
							js_command = "document.getElementById('Du1_Select').options[document.getElementById('Du1_Select').options.length]=new Option('#{diametr}','#{diametr}');"
							dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
				when "set_activ_du1"
					typedoc = arr[1]
					name_doc = arr[2]
					du1 = arr[3]
					typedoc  = encode_to_utf8(typedoc)
					name_doc = encode_to_utf8(name_doc)
					du1      = encode_to_utf8(du1)
					diametrs2 = @cp_reducer_database.get_row_uniq_filter3("Тип_документа",typedoc,"Документ",name_doc,"DN1",du1,"DN2")
					if diametrs2.length>0
						diametrs2.each do |diametr|
							js_command = "document.getElementById('Du2_Select').options[document.getElementById('Du2_Select').options.length]=new Option('#{diametr}','#{diametr}');"
							dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
				when "set_activ_du2"
					typedoc = arr[1]
					name_doc = arr[2]
					du1 = arr[3]
					du2 = arr[4]
					typedoc  = encode_to_utf8(typedoc)
					name_doc = encode_to_utf8(name_doc)
					du1      = encode_to_utf8(du1)
					du2      = encode_to_utf8(du2)
					diametrs = @cp_reducer_database.get_row_uniq_filter3("Тип_документа",typedoc,"Документ",name_doc,"DN1",du1,"DN2")
					nametubes = @cp_reducer_database.get_row_uniq_filter3("Тип_документа",typedoc,"Документ",name_doc,"DN1",du1,"Наименование_перехода")
					nametube = ""
					for i in 0..diametrs.length-1
						nametube = nametubes[i] if diametrs[i]==du2
					end
					js_command = "document.getElementById('Name_Reducer').innerHTML=\"#{nametube}\";"
					dlg.execute_script(js_command) #пишем наименование трубы
				when "cancel"
					dlg.close
				when "draw_reducer" #рисуем трубопровод
					base = dlg.get_element_value("Base_Type") 
					doc = dlg.get_element_value("Document_Name") 
					du1 = dlg.get_element_value("Du1_Select") 
					du2 = dlg.get_element_value("Du2_Select") 
					massa    = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"massa")
					d1_geom  = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"D1_geom")
					d2_geom  = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"D2_geom")
					stenka1  = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"T1")
					stenka2  = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"T2")
					length   = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"L")
					namereducer = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"Наименование_перехода")
					#namereducer =$coolpipe_langDriver["Переход"]+" К-1-#{d1_geom}x#{stenka1}-#{d2_geom}x#{stenka2} #{doc}"
					param     ={:Тип            => "Переход",   #Тип компонента: Переход
								:Имя            => namereducer, #Наименование Перехода для спецификации
								:ЕдИзм          => $coolpipe_langDriver["шт"],#Единица измерения для спецификации
								:масса          => massa,       #Масса единицы для спецификации
								:D1             => d1_geom,     #Диаметр 1 наружний D1 -> D2 
								:D2             => d2_geom,     #Диаметр 2 наружний D1 -> D2 
								:Длина          => length,      #Длина перехода
								:Сегментов      => $cp_segments,#Кол-во сегметов окружности
								:стенка1        => stenka1,     #Толщина стенки трубопровода 1
								:стенка2        => stenka2,     #Толщина стенки трубопровода 2
								:ГОСТ           => doc,         #Нормативный документ (из базы)
								:Теплоизоляция  => "0",         #Толщина теплоизоляции мм, если нет то 0
								:Материал       => "0",         #Материал Перехода (собственный цвет из настроек слоев)-если нет то 0
								:typeconstruction=>"concentric" #Тип конструкции концентрический
								}
					dlg.close
					Sketchup.active_model.select_tool ToolDrawReducer.new(param) if change_diametr==false # АКТИВАЦИЯ ТУЛА ЧЕРЧЕНИЯ ПЕРЕХОДА
					$cp_change_reducer_tool.change_diam(param)                   if change_diametr==true  # Изменение диаметра
			end #case arr[0]
		end
		#########
		def cp_change_list_reduserbase_dialog               #Диалог редактирования списка баз по переходам ГОСТ/ТУ/DIN и т.д.
			if @cp_activ_changelistreducerbase_dialog==nil
			@cp_reducer_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/reducer_base.csv"))) if @cp_reducer_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список баз по переходам"], true,$coolpipe_langDriver["Список баз по переходам"], 300, 300, 100, 100, true);#width,height,left,top
			@cp_activ_changelistreducerbase_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/change_list_pipebase_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			if params==nil
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_change_list_reduserbase_dialog(arr,dlg) #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 300
			dlg.max_width  = 300
			dlg.min_height = 300
			dlg.min_width  = 300
			dlg.set_on_close{@cp_activ_changelistreducerbase_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_activ_changelistreducerbase_dialog==nil
		end #cp_change_list_reduserbase_dialog
		def check_commands_change_list_reduserbase_dialog(arr,dlg)
			case arr[0]
				when "listbase_load_succesfull" #если окно загруженно заполняем его
					typedocs = @cp_reducer_database.get_row_uniq("Тип_документа")
					if typedocs.length>0
					typedocs.each do |typedoc|
					if (typedoc!="") && (typedoc!=nil)
					js_command = "addrow('#{typedoc}')"
					dlg.execute_script(js_command) #запоняем список базы данных
					end
					end
					end
					#Устанавливаем текущие языковые параметры
					 dlg.execute_script("document.getElementById('text_editbases').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка баз"]}\"")
					 dlg.execute_script("document.getElementById('text_typedocuments').innerHTML=\"#{$coolpipe_langDriver["Тип документов"]}\"")
					 dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно_удалить:"]}\"")
					 dlg.execute_script("document.getElementById('newbase').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					 dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					 dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					 dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistbase"
					#Редактируем список с базами
					for i in 1..(arr.length-1)
						@cp_reducer_database.add_newtypedoc(encode_to_utf8(arr[i]))
					end
					dlg.close
					cp_refreshdialog(@cp_activ_selectreducer_dialog,"selectreducer_dialog")
				when "delete"
					@cp_reducer_database.delete_typedoc(encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selectreducer_dialog,"selectreducer_dialog")
			end
		end
		#########
		def cp_change_list_reduser_documents_dialog(base)   #Диалог редактирования списка документов переходов
			if @cp_activ_changelistreducerdocuments_dialog==nil
			@cp_reducer_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/reducer_base.csv"))) if @cp_reducer_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список документов по переходам"], true,$coolpipe_langDriver["Список документов по переходам"], 550, 400, 100, 100, true);#width,height,left,top
			@cp_activ_changelistreducerdocuments_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_pipe_document_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			if params==nil
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_change_list_reduser_documents_dialog(arr,dlg,base)#Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 400
			dlg.max_width  = 550
			dlg.min_height = 400
			dlg.min_width  = 550
			dlg.set_on_close{@cp_activ_changelistreducerdocuments_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_activ_changelistpipedocuments_dialog==nil
		end #cp_change_list_reduser_documents_dialog
		def check_commands_change_list_reduser_documents_dialog(arr,dlg,base)
			case arr[0]
				when "listdocuments_load_succesfull" #если окно загруженно заполняем его
					namedocs = @cp_reducer_database.get_row_uniq_filter1("Тип_документа",base,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
						if (namedoc!="") && (namedoc!=nil)
						descriptdoc = @cp_reducer_database.get_descript_doc("Документ",namedoc,"Описание_документа")
						js_command = "addrow(\"#{namedoc}\",\"#{descriptdoc}\")"
						dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
					end
					#Устанавливаем текущие языковые параметры
					 dlg.execute_script("document.getElementById('text_editdoclist').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка документов"]}\"")
					 dlg.execute_script("document.getElementById('text_namedoc').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
					 dlg.execute_script("document.getElementById('text_descriptdoc').innerHTML=\"#{$coolpipe_langDriver["Описание документа"]}\"")
					 dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					 dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					 dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					 dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					 dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "skp:ValueChanged@changelistdocuments"
					dlg.close
				when "changelistdocuments"
					#Редактируем список с базами
					dlg.close
					if arr[1]!="0"
						for i in 1..(arr.length-1)
							arr2 = arr[i].split("=")
							@cp_reducer_database.add_newdoc(base,encode_to_utf8(arr2[0]),encode_to_utf8(arr2[1]))
						end
						cp_refreshdialog(@cp_activ_selectreducer_dialog,"selectreducer_dialog")
					end
				when "delete"
					@cp_reducer_database.delete_doc(base,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selectreducer_dialog,"selectreducer_dialog")
			end
		end
		#########
		def cp_change_list_redusers(base,doc)               #Диалог редактирования списка элементов переходов в соотв. документе
			if @cp_change_list_reducers_dialog==nil
			@cp_reducer_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/reducer_base.csv"))) if @cp_reducer_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список элементов в документе "]+doc, true,$coolpipe_langDriver["Список элементов в документе "]+doc, 800, 400, 100, 100, true);#width,height,left,top
			@cp_change_list_reducers_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_reducer_element_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			enc = true
			if (params==nil) or (params=="changelistreducerselements|readalt")
				enc = false if (params=="changelistreducerselements|readalt")
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_change_list_redusers(arr,dlg,base,doc) #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 400
			dlg.max_width  = 800
			dlg.min_height = 400
			dlg.min_width  = 800
			dlg.set_on_close{@cp_change_list_reducers_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			
			end #if @cp_change_list_reducers_dialog==nil
		end #cp_change_list_redusers(base,doc)  
		def check_commands_change_list_redusers(arr,dlg,base,doc)
			case arr[0]
				when "listdu_load_succesfull" #если окно загруженно заполняем его
					namedu1 = @cp_reducer_database.get_row_uniq_filter2("Тип_документа",base,"Документ",doc,"DN1")
					if namedu1.length>0
						namedu1.each do |du1|
							namedu2 = @cp_reducer_database.get_row_uniq_filter3("Тип_документа",base,"Документ",doc,"DN1",du1,"DN2")
							namedu2.each do |du2|
								if (du2!="") && (du2!=nil) #addrow(du,dn,st,mas,name)
									namedn1 = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"D1_geom")
									namedn2 = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"D2_geom")
									namest1 = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"T1")
									namest2 = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"T2")
									dlina   = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"L")
									namemas = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"massa")
									namedescript = @cp_reducer_database.get_reducer_params(base,doc,du1,du2,"Наименование_перехода")
									js_command = "addrow(\"#{du1}\",\"#{du2}\",\"#{namedn1}\",\"#{namedn2}\",\"#{namest1}\",\"#{namest2}\",\"#{dlina}\",\"#{namemas}\",\"#{namedescript}\")"
									dlg.execute_script(js_command) #запоняем список базы данных
								end
							end #namedu2.each do |du2|
					end #namedu1.each do |du1|
					end #if namedu1.length>0
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_editlistreducers').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка переходов"]}\"")
					dlg.execute_script("document.getElementById('text_DiametrUslov1').innerHTML=\"#{$coolpipe_langDriver["Диаметр условный"]} 1\"")
					dlg.execute_script("document.getElementById('text_DiametrUslov2').innerHTML=\"#{$coolpipe_langDriver["Диаметр условный"]} 2\"")
					dlg.execute_script("document.getElementById('text_DiametrNar1').innerHTML=\"#{$coolpipe_langDriver["Диаметр наружний"]} 1\"")
					dlg.execute_script("document.getElementById('text_DiametrNar2').innerHTML=\"#{$coolpipe_langDriver["Диаметр наружний"]} 2\"")
					dlg.execute_script("document.getElementById('text_Stenka1').innerHTML=\"#{$coolpipe_langDriver["Толщина стенки"]} 1\"")
					dlg.execute_script("document.getElementById('text_Stenka2').innerHTML=\"#{$coolpipe_langDriver["Толщина стенки"]} 2\"")
					dlg.execute_script("document.getElementById('text_LengthReducer').innerHTML=\"#{$coolpipe_langDriver["Длина<br>[мм]"]}\"")
					dlg.execute_script("document.getElementById('text_MassaReducer').innerHTML=\"#{$coolpipe_langDriver["Масса"]}\"")
					dlg.execute_script("document.getElementById('text_NameForSpec').innerHTML=\"#{$coolpipe_langDriver["Наименование для спецификации"]}\"")
					dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistreducerselements"
					#Редактируем список с базами
					@cp_change_list_reducers_dialog.close
					if arr[1]!="0"
						listdu=[]
						for i in 1..arr.length-1
							listdu<<encode_to_utf8(arr[i])
						end
						@cp_reducer_database.add_tubes(base,doc,listdu)
						cp_refreshdialog(@cp_activ_selectreducer_dialog,"selectreducer_dialog")
					end
				when "delete"
					@cp_reducer_database.delete_tube(base,doc,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selectreducer_dialog,"selectreducer_dialog")
			end
		end
		##################################################################################################
		#------- Тройники
		##################################################################################################
		def cp_selecttee_dialog(change_diametr = false)     #Диалог выбора тройников
			if @cp_activ_selecttee_dialog==nil
			dialog_ini = File.join(Sketchup.find_support_file("Plugins/CoolPipe/ini/tee_dialog.ini"))
			@cp_tee_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/tee_base.csv")))
			dlg = UI::WebDialog.new($coolpipe_langDriver["Выбор тройника"], true,$coolpipe_langDriver["Выбор тройника"], 475, 450, 100, 100, true);#width,height,left,top
			@cp_activ_selecttee_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/select_tee_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
				if params==nil
					params = dlg.get_element_value("Alt_CallBack") 
				end
				arr=params.split("|")
				check_commands_selecttee_dialog(arr,dlg,change_diametr,dialog_ini) #Проверка и обработка комманд поступающих от диалогового окна
			}   #dlg.add_action_callback("ValueChanged") {|dialog, params|
			dlg.max_height = 450
			dlg.max_width  = 600
			dlg.min_height = 450
			dlg.min_width  = 500
			dlg.set_on_close{
			#сохранение последнего состояния окна
			ss_base = dlg.get_element_value("Base_Type") 
			ss_doc = dlg.get_element_value("Document_Name") 
			ss_du1 = dlg.get_element_value("Du1_Select") 
			ss_du2 = dlg.get_element_value("Du2_Select") 

			text = ss_base+"|"+ss_doc+"|"+ss_du1+"|"+ss_du2
			File.open(dialog_ini, "w") do |file|
				file.puts text 
			end
			@cp_activ_selecttee_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_activ_selectreducer_dialog==nil
		end
		def check_commands_selecttee_dialog(arr,dlg,change_diametr,dialog_ini)
			case arr[0]
				when "load_succesfull"
					typedocs = @cp_tee_database.get_row_uniq("Тип_документа")
					typedocs.each do |typedoc|
					typedoc = typedoc.chomp
					js_command = "document.getElementById('Base_Type').options[document.getElementById('Base_Type').options.length]=new Option('#{typedoc}','#{typedoc}');"
					dlg.execute_script(js_command) #запоняем список базы данных
					end
					#восстанавливаем последнее состояние диалога
					File.open(dialog_ini).each do |line|
					line = line.chomp
					if (line!="") && (line!=" ") && (line!=nil)
					restore = line.split("|")
					dlg.execute_script("document.getElementById('Base_Type').value=\"#{restore[0]}\"")
					dlg.execute_script("fsetactivebase()")
					dlg.execute_script("document.getElementById('Document_Name').value=\"#{restore[1]}\"")
					dlg.execute_script("fsetactivedocument()")
					dlg.execute_script("document.getElementById('Du1_Select').value=\"#{restore[2]}\"")
					dlg.execute_script("fsetactivedu1()")
					dlg.execute_script("document.getElementById('Du2_Select').value=\"#{restore[3]}\"")
					dlg.execute_script("fsetactivedu2()")
					end
					end
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_database').innerHTML=\"#{$coolpipe_langDriver["База данных"]}\"")
					dlg.execute_script("document.getElementById('text_document').innerHTML=\"#{$coolpipe_langDriver["Документ:"]}\"")
					dlg.execute_script("document.getElementById('text_nameofdocument').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
					dlg.execute_script("document.getElementById('text_DN1').innerHTML=\"#{$coolpipe_langDriver["Ду"]}1\"")
					dlg.execute_script("document.getElementById('text_DN2').innerHTML=\"#{$coolpipe_langDriver["Ду"]}2\"")
					dlg.execute_script("document.getElementById('text_nametee').innerHTML=\"#{$coolpipe_langDriver["Наименование:"]}\"")
					dlg.execute_script("document.getElementById('text_draw').value=\"#{$coolpipe_langDriver["Чертить"]}\"")
					dlg.execute_script("document.getElementById('text_cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "change_list_base"
					cp_change_list_teebase_dialog
				when "change_list_document"
					base = encode_to_utf8(arr[1])
					cp_change_list_tee_documents_dialog(base)
				when "change_list_reducers"
					base = encode_to_utf8(arr[1])
					doc  = encode_to_utf8(arr[2])
					cp_change_list_tees(base,doc)
				when "set_activ_base"
					name_filter = encode_to_utf8(arr[1])
					namedocs = @cp_tee_database.get_row_uniq_filter1("Тип_документа",name_filter,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
							if (namedoc!="") && (namedoc!=nil)
								js_command = "document.getElementById('Document_Name').options[document.getElementById('Document_Name').options.length]=new Option('#{namedoc}','#{namedoc}');"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
				when "set_activ_document"
					typedoc = encode_to_utf8(arr[1])
					name_filter = encode_to_utf8(arr[2])
					diametrs = @cp_tee_database.get_row_uniq_filter2("Тип_документа",typedoc,"Документ",name_filter,"Ду1")
					descriptdoc = @cp_tee_database.get_descript_doc("Документ",name_filter,"Описание_документа")
					js_command = "document.getElementById('Name_DOC').innerHTML=\"#{descriptdoc}\";"
					dlg.execute_script(js_command) #показываем описание документа
					if diametrs.length>0
						diametrs.each do |diametr|
							js_command = "document.getElementById('Du1_Select').options[document.getElementById('Du1_Select').options.length]=new Option('#{diametr}','#{diametr}');"
							dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
				when "set_activ_du1"
					typedoc = encode_to_utf8(arr[1])
					name_doc = encode_to_utf8(arr[2])
					du1 = arr[3]
					diametrs2 = @cp_tee_database.get_row_uniq_filter3("Тип_документа",typedoc,"Документ",name_doc,"Ду1",du1,"Ду2")
					if diametrs2.length>0
						diametrs2.each do |diametr|
							js_command = "document.getElementById('Du2_Select').options[document.getElementById('Du2_Select').options.length]=new Option('#{diametr}','#{diametr}');"
							dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
				when "set_activ_du2"
					typedoc = encode_to_utf8(arr[1])
					name_doc = encode_to_utf8(arr[2])
					du1 = encode_to_utf8(arr[3])
					du2 = encode_to_utf8(arr[4])
					diametrs = @cp_tee_database.get_row_uniq_filter3("Тип_документа",typedoc,"Документ",name_doc,"Ду1",du1,"Ду2")
					nametubes = @cp_tee_database.get_row_uniq_filter3("Тип_документа",typedoc,"Документ",name_doc,"Ду1",du1,"Наименование_перехода")
					nametube = ""
					for i in 0..diametrs.length-1
						nametube = nametubes[i] if diametrs[i]==du2
					end
					js_command = "document.getElementById('Name_Tee').innerHTML=\"#{nametube}\";"
					dlg.execute_script(js_command) #пишем наименование трубы
				when "cancel"
					dlg.close
				when "draw_tee" #рисуем тройник
					base = dlg.get_element_value("Base_Type") 
					doc = dlg.get_element_value("Document_Name") 
					du1 = dlg.get_element_value("Du1_Select") 
					du2 = dlg.get_element_value("Du2_Select") 
					massa    = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Масса")
					d1_geom  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Дн1")
					d2_geom  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Дн2")
					stenka1  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Стенка1")
					stenka2  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Стенка2")
					length1  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"L1")
					length2  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"L2")
					nametee  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Наименование_тройника")
					#nametee =$coolpipe_langDriver["Тройник"]+" #{d1_geom}x#{stenka1}-#{d2_geom}x#{stenka2}-#{d1_geom}x#{stenka1} #{doc}"
					param    = {:Тип            => "Тройник",   #Тип компонента: Тройник
								:Имя            => nametee,     #Наименование Тройника для спецификации
								:ЕдИзм          => $coolpipe_langDriver["шт"], #Единица измерения для спецификации
								:масса          => massa,       #Масса единицы для спецификации
								:D1             => d1_geom,     #Диаметр 1 наружний D1 х D2 х D1
								:D2             => d2_geom,     #Диаметр 2 наружний D1 х D2 х D1
								:Dнар           => d1_geom,     #Диаметр 1 наружний D1 х D2 х D1
								:Длина          => length1,     #Длина тройника по большему диаметру
								:Высота         => length2,     #Длина тройника по меньшему диаметру
								:стенка1        => stenka1,     #Толщина стенки трубопровода 1
								:стенка2        => stenka2,     #Толщина стенки трубопровода 2
								:ГОСТ           => doc,         #Нормативный документ (из базы)
								:Теплоизоляция  => "0",         #Толщина теплоизоляции мм, если нет то 0
								:Материал       => "0",         #Материал Тройника (собственный цвет из настроек слоев)-если нет то 0
								}
					dlg.close
					Sketchup.active_model.select_tool ToolDrawTee.new(param) if change_diametr==false # АКТИВАЦИЯ ТУЛА ЧЕРЧЕНИЯ тройника
					$cp_change_tee_tool.change_diam(param)                   if change_diametr==true  # Изменение диаметра
			end #case arr[0]
		end
		#########
		def cp_change_list_teebase_dialog                   #Диалог редактирования списка баз по тройникам ГОСТ/ТУ/DIN и т.д.
			if @cp_activ_changelistteebase_dialog==nil
			@cp_tee_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/tee_base.csv"))) if @cp_tee_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список баз по тройникам"], true,$coolpipe_langDriver["Список баз по тройникам"], 300, 300, 100, 100, true);#width,height,left,top
			@cp_activ_changelistteebase_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/change_list_pipebase_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
				if params==nil
					params = dlg.get_element_value("Alt_CallBack") 
				end
				arr=params.split("|")
				check_commands_change_list_teebase_dialog(arr,dlg)#Проверка и обработка комманд поступающих от диалогового окна
			} 
			dlg.max_height = 300
			dlg.max_width  = 300
			dlg.min_height = 300
			dlg.min_width  = 300
			dlg.set_on_close{@cp_activ_changelistteebase_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_activ_changelistteebase_dialog==nil
		end #cp_change_list_teebase_dialog
		def check_commands_change_list_teebase_dialog(arr,dlg)
			case arr[0]
				when "listbase_load_succesfull" #если окно загруженно заполняем его
					typedocs = @cp_tee_database.get_row_uniq("Тип_документа")
					if typedocs.length>0
						typedocs.each do |typedoc|
							if (typedoc!="") && (typedoc!=nil)
								js_command = "addrow('#{typedoc}')"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
					#Устанавливаем текущие языковые параметры
					 dlg.execute_script("document.getElementById('text_editbases').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка баз"]}\"")
					 dlg.execute_script("document.getElementById('text_typedocuments').innerHTML=\"#{$coolpipe_langDriver["Тип документов"]}\"")
					 dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					 dlg.execute_script("document.getElementById('newbase').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					 dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					 dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					 dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistbase"
					#Редактируем список с базами
					for i in 1..(arr.length-1)
						@cp_tee_database.add_newtypedoc(encode_to_utf8(arr[i]))
					end
					dlg.close
					cp_refreshdialog(@cp_activ_selecttee_dialog,"selecttee_dialog")
				when "delete"
					@cp_tee_database.delete_typedoc(encode_to_utf8(arr[1]))
			end
		end
		#########
		def cp_change_list_tee_documents_dialog(base)       #Диалог редактирования списка документов тройников
			if @cp_activ_changelistteedocuments_dialog==nil
			@cp_tee_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/tee_base.csv"))) if @cp_tee_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список документов по тройникам"], true,$coolpipe_langDriver["Список документов по тройникам"], 550, 400, 100, 100, true);#width,height,left,top
			@cp_activ_changelistteedocuments_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_pipe_document_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			if params==nil
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_change_list_tee_documents_dialog(arr,dlg,base) #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 400
			dlg.max_width  = 550
			dlg.min_height = 400
			dlg.min_width  = 550
			dlg.set_on_close{@cp_activ_changelistteedocuments_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_activ_changelistteedocuments_dialog==nil
		end #cp_change_list_tee_documents_dialog
		def check_commands_change_list_tee_documents_dialog(arr,dlg,base)
			case arr[0]
				when "listdocuments_load_succesfull" #если окно загруженно заполняем его
					namedocs = @cp_tee_database.get_row_uniq_filter1("Тип_документа",base,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
						if (namedoc!="") && (namedoc!=nil)
						descriptdoc = @cp_tee_database.get_descript_doc("Документ",namedoc,"Описание_документа")
						js_command = "addrow(\"#{namedoc}\",\"#{descriptdoc}\")"
						dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
					end
					#Устанавливаем текущие языковые параметры
					 dlg.execute_script("document.getElementById('text_editdoclist').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка документов"]}\"")
					 dlg.execute_script("document.getElementById('text_namedoc').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
					 dlg.execute_script("document.getElementById('text_descriptdoc').innerHTML=\"#{$coolpipe_langDriver["Описание документа"]}\"")
					 dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					 dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					 dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					 dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					 dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "skp:ValueChanged@changelistdocuments"
					dlg.close
				when "changelistdocuments"
					#Редактируем список с базами
					dlg.close
					if arr[1]!="0"
						for i in 1..(arr.length-1)
							arr2 = arr[i].split("=")
							@cp_tee_database.add_newdoc(base,encode_to_utf8(arr2[0]),encode_to_utf8(arr2[1]))
						end
						cp_refreshdialog(@cp_activ_selecttee_dialog,"selecttee_dialog")
					end
				when "delete"
					@cp_tee_database.delete_doc(base,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selecttee_dialog,"selecttee_dialog")
			end
		end
		#########
		def cp_change_list_tees(base,doc)                   #Диалог редактирования списка элементов тройников в соотв. документе
			if @cp_change_list_tees_dialog==nil
			@cp_tee_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/tee_base.csv"))) if @cp_tee_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список элементов в документе"]+doc, true,$coolpipe_langDriver["Список элементов в документе"]+doc, 880, 400, 100, 100, true);#width,height,left,top
			@cp_change_list_tees_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_tee_element_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			enc = true
			if (params==nil) or (params=="changelistreducerselements|readalt")
				enc = false if (params=="changelistreducerselements|readalt")
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
				check_commands_change_list_tees(arr,dlg,base,doc) #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 400
			dlg.max_width  = 880
			dlg.min_height = 400
			dlg.min_width  = 880
			dlg.set_on_close{@cp_change_list_tees_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_change_list_tees_dialog==nil
		end #cp_change_list_tees
		def check_commands_change_list_tees(arr,dlg,base,doc)
			case arr[0]
				when "listdu_load_succesfull" #если окно загруженно заполняем его
					namedu1 = @cp_tee_database.get_row_uniq_filter2("Тип_документа",base,"Документ",doc,"Ду1")
					if namedu1.length>0
						namedu1.each do |du1|
							namedu2 = @cp_tee_database.get_row_uniq_filter3("Тип_документа",base,"Документ",doc,"Ду1",du1,"Ду2")
							namedu2.each do |du2|
								if (du2!="") && (du2!=nil) #addrow(du,dn,st,mas,name)
									namedn1 = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Дн1")
									namedn2 = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Дн2")
									dlina1  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"L1")
									dlina2  = @cp_tee_database.get_tee_params(base,doc,du1,du2,"L2")
									namest1 = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Стенка1")
									namest2 = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Стенка2")
									namemas = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Масса")
									namedescript = @cp_tee_database.get_tee_params(base,doc,du1,du2,"Наименование_тройника")
									js_command = "addrow(\"#{du1}\",\"#{du2}\",\"#{namedn1}\",\"#{namedn2}\",\"#{dlina1}\",\"#{dlina2}\",\"#{namest1}\",\"#{namest2}\",\"#{namemas}\",\"#{namedescript}\")"
									dlg.execute_script(js_command) #запоняем список базы данных
								end
							end #namedu2.each do |du2|
						end #namedu1.each do |du1|
					end #if namedu1.length>0
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_editlisttees').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка тройников"]}\"")
					dlg.execute_script("document.getElementById('text_DiametrUslov1').innerHTML=\"#{$coolpipe_langDriver["Диаметр условный"]} 1\"")
					dlg.execute_script("document.getElementById('text_DiametrUslov2').innerHTML=\"#{$coolpipe_langDriver["Диаметр условный"]} 2\"")
					dlg.execute_script("document.getElementById('text_DiametrNar1').innerHTML=\"#{$coolpipe_langDriver["Диаметр наружний"]} 1\"")
					dlg.execute_script("document.getElementById('text_DiametrNar2').innerHTML=\"#{$coolpipe_langDriver["Диаметр наружний"]} 2\"")
					dlg.execute_script("document.getElementById('text_Stenka1').innerHTML=\"#{$coolpipe_langDriver["Толщина стенки"]} 1\"")
					dlg.execute_script("document.getElementById('text_Stenka2').innerHTML=\"#{$coolpipe_langDriver["Толщина стенки"]} 2\"")
					dlg.execute_script("document.getElementById('text_LengthTee1').innerHTML=\"#{$coolpipe_langDriver["Длина [мм]"]} 1\"")
					dlg.execute_script("document.getElementById('text_LengthTee2').innerHTML=\"#{$coolpipe_langDriver["Длина [мм]"]} 2\"")
					dlg.execute_script("document.getElementById('text_MassaTee').innerHTML=\"#{$coolpipe_langDriver["Масса"]}\"")
					dlg.execute_script("document.getElementById('text_NameForSpec').innerHTML=\"#{$coolpipe_langDriver["Наименование для спецификации"]}\"")
					dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistreducerselements"
					#Редактируем список с базами
					@cp_change_list_tees_dialog.close
					if arr[1]!="0"
						listdu=[]
						for i in 1..arr.length-1 
							listdu<<encode_to_utf8(arr[i])
						end
						@cp_tee_database.add_tubes(base,doc,listdu)
						cp_refreshdialog(@cp_activ_selecttee_dialog,"selecttee_dialog")
					end
				when "delete"
					@cp_tee_database.delete_tube(base,doc,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selecttee_dialog,"selecttee_dialog")
			end
		end
		##################################################################################################
		#------- Фланцы
		##################################################################################################
		def cp_selectflange_dialog(change_diametr = false)  #Диалог выбора Фланцев
			if @cp_activ_selectflange_dialog==nil
				dialog_ini = File.join(Sketchup.find_support_file("Plugins/CoolPipe/ini/flange_dialog.ini"))
				@cp_flange_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/flange_base.csv")))
				dlg = UI::WebDialog.new($coolpipe_langDriver["Выбор фланца"], true,$coolpipe_langDriver["Выбор фланца"], 475, 450, 100, 100, true);#width,height,left,top
				@cp_activ_selectflange_dialog = dlg #глобальная переменная доступа к активному диалогу
				path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/select_flange_dialog.html"))
				dlg.set_file(path)
				dlg.set_background_color(dlg.get_default_dialog_color)
				dlg.add_action_callback("ValueChanged") {|dialog, params|
					if params==nil
						params = dlg.get_element_value("Alt_CallBack") 
					end
					arr=params.split("|")
					check_commands_selectflange_dialog(arr,dlg,change_diametr,dialog_ini) #Проверка и обработка комманд поступающих от диалогового окна
				} # end dlg.add_action_callback
				dlg.max_height = 450
				dlg.max_width  = 600
				dlg.min_height = 450
				dlg.min_width  = 500
				dlg.set_on_close{
				#сохранение последнего состояния окна
				ss_base = dlg.get_element_value("Base_Type") 
				ss_doc = dlg.get_element_value("Document_Name") 
				ss_du = dlg.get_element_value("Du_Select") 
				text = ss_base+"|"+ss_doc+"|"+ss_du
				File.open(dialog_ini, "w") do |file|
					file.puts text 
				end
				@cp_activ_selectflange_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
				dlg.show
			end #if @cp_activ_selectpipe_dialog==nil
		end #def cp_selectpipe_dialog
		def check_commands_selectflange_dialog(arr,dlg,change_diametr,dialog_ini)
			case arr[0]
				###########
				#если окно загруженно заполняем его    
				when "load_succesfull"     
					typedocs = @cp_flange_database.get_row_uniq("Тип_документа")
						typedocs.each do |typedoc|
						typedoc = typedoc.chomp
						js_command = "document.getElementById('Base_Type').options[document.getElementById('Base_Type').options.length]=new Option('#{typedoc}','#{typedoc}');"
						dlg.execute_script(js_command) #запоняем список базы данных
					end
					#восстановление предыдущего состояния
					File.open(dialog_ini).each do |line|
						line = line.chomp
						if (line!="") && (line!=" ") && (line!=nil)
							restore = line.split("|")
							dlg.execute_script("document.getElementById('Base_Type').value=\"#{restore[0]}\"")
							dlg.execute_script("fsetactivebase()")
							dlg.execute_script("document.getElementById('Document_Name').value=\"#{restore[1]}\"")
							dlg.execute_script("fsetactivedocument()")
							dlg.execute_script("document.getElementById('Du_Select').value=\"#{restore[2]}\"")
							dlg.execute_script("fsetactivedu()")
						end
					end
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_database').innerHTML=\"#{$coolpipe_langDriver["База данных"]}\"")
					dlg.execute_script("document.getElementById('text_document').innerHTML=\"#{$coolpipe_langDriver["Документ:"]}\"")
					dlg.execute_script("document.getElementById('text_nameofdocument').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
					dlg.execute_script("document.getElementById('text_uslovndiametr').innerHTML=\"#{$coolpipe_langDriver["Условный диаметр:"]}\"")
					dlg.execute_script("document.getElementById('text_nametube').innerHTML=\"#{$coolpipe_langDriver["Наименование:"]}\"")
					dlg.execute_script("document.getElementById('text_draw').value=\"#{$coolpipe_langDriver["Чертить"]}\"")
					dlg.execute_script("document.getElementById('text_cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				###########
				#выбрана выбран активный тип документов (например ГОСТ)
				when "set_activ_base" 
					name_filter = encode_to_utf8(arr[1])
					namedocs = @cp_flange_database.get_row_uniq_filter1("Тип_документа",name_filter,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
							if (namedoc!="") && (namedoc!=nil)
								js_command = "document.getElementById('Document_Name').options[document.getElementById('Document_Name').options.length]=new Option('#{namedoc}','#{namedoc}');"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
				###########
				#выбран активный документ из базы
				when "set_activ_document" 
					typedoc = encode_to_utf8(arr[1])
					name_filter = encode_to_utf8(arr[2])
					diametrs = @cp_flange_database.get_row_uniq_filter2("Тип_документа",typedoc,"Документ",name_filter,"Ду")
					descriptdoc = @cp_flange_database.get_descript_doc("Документ",name_filter,"Описание_документа")
					js_command = "document.getElementById('Name_DOC').innerHTML=\"#{descriptdoc}\";"
					dlg.execute_script(js_command) #показываем описание документа
					if diametrs.length>0
						diametrs.each do |diametr|
							js_command = "document.getElementById('Du_Select').options[document.getElementById('Du_Select').options.length]=new Option('#{diametr}','#{diametr}');"
							dlg.execute_script(js_command) #запоняем список базы данных
						end
					end
				###########
				when "set_activ_du" #выбран условный диаметр из базы
					typedoc = encode_to_utf8(arr[1])
					docname = encode_to_utf8(arr[2])
					du = encode_to_utf8(arr[3])
					nametube = @cp_flange_database.get_descript_element("Документ",docname,"Ду",du,"Наименование_фланца")
					js_command = "document.getElementById('Name_Flange').innerHTML=\"#{nametube}\";"
					dlg.execute_script(js_command) #пишем наименование трубы
				###########
				when "change_list_base"
					cp_change_list_flangebase_dialog
				when "change_list_document"
					base = encode_to_utf8(arr[1])
					cp_change_list_flange_documents_dialog(base)
				when "change_list_pipes_du"
					base = encode_to_utf8(arr[1])
					doc = encode_to_utf8(arr[2])
					cp_change_list_flanges(base,doc)
				when "cancel"
					dlg.close
				when "draw_flange" #рисуем трубопровод
					base = dlg.get_element_value("Base_Type") 
					doc = dlg.get_element_value("Document_Name") 
					du = dlg.get_element_value("Du_Select") 
					du       = @cp_flange_database.get_flange_params(base,doc,du,"Ду")
					d        = @cp_flange_database.get_flange_params(base,doc,du,"d")
					n        = @cp_flange_database.get_flange_params(base,doc,du,"n")
					d1       = @cp_flange_database.get_flange_params(base,doc,du,"D1")
					d2       = @cp_flange_database.get_flange_params(base,doc,du,"D2")
					d3       = @cp_flange_database.get_flange_params(base,doc,du,"D3")
					d4       = @cp_flange_database.get_flange_params(base,doc,du,"D4")
					d5       = @cp_flange_database.get_flange_params(base,doc,du,"D5") #Расположение болтового отверстия - при содействии Nemesis
					h1       = @cp_flange_database.get_flange_params(base,doc,du,"h1")
					h2       = @cp_flange_database.get_flange_params(base,doc,du,"h2")
					h3       = @cp_flange_database.get_flange_params(base,doc,du,"h3")
					nameflange = @cp_flange_database.get_flange_params(base,doc,du,"Наименование_трубопровода")
					#nameflange =$coolpipe_langDriver["Фланец"]+" Ду=#{du} #{doc}"
					param={:Тип           => "Фланец",     #Тип компонента: Фланец
						   :База           => base,         #Тип базы ГОСТ, ТУ или др...
						   :ГОСТ          => doc,          #Нормативный документ (из базы)
						   :Ду              => du,           #Диаметр трубопровода условный
						   :Имя            => nameflange,   #Наименование трубы для спецификации
						   :ЕдИзм          => $coolpipe_langDriver["шт"], #Единица измерения для спецификации
						   :масса          => "?",          #Масса единицы для спецификации
						   :Dнар           => d1,           #Диаметр трубопровода наружний
						   :Cегментов    => $cp_segments,
						   :D1              => d1,
						   :D2              => d2,
						   :D3              => d3,
						   :D4              => d4,
						   :D5				=> d5, #Расположение болтового отверстия - при содействии Nemesis
						   :h1              => h1,
						   :h2              => h2,
						   :h3              => h3,
						   :n_отв          => n,
						   :d_отв          => d
							}
					dlg.close
					Sketchup.active_model.select_tool ToolDrawFlange.new(param) if change_diametr==false #АКТИВАЦИЯ ТУЛА ЧЕРЧЕНИЯ Фланца
			end
		end
		#########
		def cp_change_list_flangebase_dialog                #Диалог редактирования списка баз по фланцам ГОСТ/ТУ/DIN и т.д.
			if @cp_activ_changelistflangebase_dialog==nil
			@cp_flange_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/flange_base.csv"))) if @cp_flange_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список баз по фланцам"], true,$coolpipe_langDriver["Список баз по фланцам"], 300, 300, 100, 100, true);#width,height,left,top
			@cp_activ_changelistflangebase_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/change_list_pipebase_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
			if params==nil
				params = dlg.get_element_value("Alt_CallBack") 
			end
			arr=params.split("|")
			check_commands_change_list_flangebase_dialog(arr,dlg) #Проверка и обработка комманд поступающих от диалогового окна
			}
			dlg.max_height = 300
			dlg.max_width  = 300
			dlg.min_height = 300
			dlg.min_width  = 300
			dlg.set_on_close{@cp_activ_changelistflangebase_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_activ_changelistflangebase_dialog==nil
		end #cp_change_list_flangebase_dialog
		def check_commands_change_list_flangebase_dialog(arr,dlg)
			case arr[0]
				when "listbase_load_succesfull" #если окно загруженно заполняем его
					typedocs = @cp_flange_database.get_row_uniq("Тип_документа")
					if typedocs.length>0
						typedocs.each do |typedoc|
							if (typedoc!="") && (typedoc!=nil)
								js_command = "addrow('#{typedoc}')"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
					#Устанавливаем текущие языковые параметры
					 dlg.execute_script("document.getElementById('text_editbases').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка баз"]}\"")
					 dlg.execute_script("document.getElementById('text_typedocuments').innerHTML=\"#{$coolpipe_langDriver["Тип документов"]}\"")
					 dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить"]}\"")
					 dlg.execute_script("document.getElementById('newbase').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					 dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					 dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					 dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistbase"
					#Редактируем список с базами
					for i in 1..(arr.length-1)
						@cp_flange_database.add_newtypedoc(encode_to_utf8(arr[i]))
					end
					dlg.close
					cp_refreshdialog(@cp_activ_selectflange_dialog,"selectflange_dialog")
				when "delete"
					@cp_flange_database.delete_typedoc(encode_to_utf8(arr[1]))
			end
		end
		#########
		def cp_change_list_flange_documents_dialog(base)    #Диалог редактирования списка документов фланцев
			if @cp_activ_changelistflangedocuments_dialog==nil
				@cp_flange_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/flange_base.csv"))) if @cp_flange_database==nil #Загружаем базу если она не загруженна
				dlg = UI::WebDialog.new($coolpipe_langDriver["Список документов по фланцам"], true,$coolpipe_langDriver["Список документов по фланцам"], 550, 400, 100, 100, true);#width,height,left,top
				@cp_activ_changelistflangedocuments_dialog = dlg #глобальная переменная доступа к активному диалогу
				path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_pipe_document_dialog.html"))
				dlg.set_file(path)
				dlg.set_background_color(dlg.get_default_dialog_color)
				dlg.add_action_callback("ValueChanged") {|dialog, params|
					if params==nil
						params = dlg.get_element_value("Alt_CallBack") 
					end
					arr=params.split("|")
					check_commands_change_list_flange_documents_dialog(arr,dlg,base) #Проверка и обработка комманд поступающих от диалогового окна
				}
				dlg.max_height = 400
				dlg.max_width  = 550
				dlg.min_height = 400
				dlg.min_width  = 550
				dlg.set_on_close{@cp_activ_changelistflangedocuments_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
				dlg.show
			end #if @cp_activ_changelistteedocuments_dialog==nil
		end
		def check_commands_change_list_flange_documents_dialog(arr,dlg,base)			
			case arr[0]
				when "listdocuments_load_succesfull" #если окно загруженно заполняем его
					namedocs = @cp_flange_database.get_row_uniq_filter1("Тип_документа",base,"Документ")
					if namedocs.length>0
						namedocs.each do |namedoc|
							if (namedoc!="") && (namedoc!=nil)
								descriptdoc = @cp_flange_database.get_descript_doc("Документ",namedoc,"Описание_документа")
								js_command = "addrow(\"#{namedoc}\",\"#{descriptdoc}\")"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
					#Устанавливаем текущие языковые параметры
					 dlg.execute_script("document.getElementById('text_editdoclist').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка документов"]}\"")
					 dlg.execute_script("document.getElementById('text_namedoc').innerHTML=\"#{$coolpipe_langDriver["Наименование документа"]}\"")
					 dlg.execute_script("document.getElementById('text_descriptdoc').innerHTML=\"#{$coolpipe_langDriver["Описание документа"]}\"")
					 dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить"]}\"")
					 dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					 dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					 dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					 dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "skp:ValueChanged@changelistdocuments"
					dlg.close
				when "changelistdocuments"
					#Редактируем список с базами
					dlg.close
					if arr[1]!="0"
						for i in 1..(arr.length-1)
							arr2 = arr[i].split("=")
							@cp_flange_database.add_newdoc(base,encode_to_utf8(arr2[0]),encode_to_utf8(arr2[1]))
						end
						cp_refreshdialog(@cp_activ_selectflange_dialog,"selectflange_dialog")
					end
				when "delete"
					@cp_flange_database.delete_doc(base,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selectflange_dialog,"selectflange_dialog")
			end
		end
		#########
		def cp_change_list_flanges(base,doc)                #Диалог редактирования списка элементов фланцев в соотв. документе
			if @cp_change_list_flanges_dialog==nil
			@cp_flange_database = CP_csv_base.new(File.join(Sketchup.find_support_file("Plugins/CoolPipe/data/flange_base.csv"))) if @cp_flange_database==nil #Загружаем базу если она не загруженна
			dlg = UI::WebDialog.new($coolpipe_langDriver["Список элементов в документе"]+doc, true,$coolpipe_langDriver["Список элементов в документе"]+doc, 930, 500, 100, 100, true);#width,height,left,top
			@cp_change_list_flanges_dialog = dlg #глобальная переменная доступа к активному диалогу
			path = File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/add_flange_element_dialog.html"))
			dlg.set_file(path)
			dlg.set_background_color(dlg.get_default_dialog_color)
			dlg.add_action_callback("ValueChanged") {|dialog, params|
				enc = true
				if (params==nil) or (params=="changelistflangeselements|readalt")
					enc = false if (params=="changelistflangeselements|readalt")
					params = dlg.get_element_value("Alt_CallBack") 
				end
				arr=params.split("|")
				check_commands_change_list_flanges(arr,dlg,base,doc)
			}
			dlg.max_height = 500
			dlg.max_width  = 930
			dlg.min_height = 500
			dlg.min_width  = 930
			dlg.set_on_close{@cp_change_list_flanges_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
			dlg.show
			end #if @cp_change_list_pipes_du_dialog==nil
		end
		def check_commands_change_list_flanges(arr,dlg,base,doc)
			case arr[0]
				when "listdu_load_succesfull" #если окно загруженно заполняем его
					namedu = @cp_flange_database.get_row_uniq_filter2("Тип_документа",base,"Документ",doc,"Ду")
					if namedu.length>0
						namedu.each do |du|
							d1 = @cp_flange_database.get_flange_params(base,doc,du,"D1")
							d2 = @cp_flange_database.get_flange_params(base,doc,du,"D2")
							d3 = @cp_flange_database.get_flange_params(base,doc,du,"D3")
							d4 = @cp_flange_database.get_flange_params(base,doc,du,"D4")
							h1 = @cp_flange_database.get_flange_params(base,doc,du,"h1")
							h2 = @cp_flange_database.get_flange_params(base,doc,du,"h2")
							h3 = @cp_flange_database.get_flange_params(base,doc,du,"h3")
							d  = @cp_flange_database.get_flange_params(base,doc,du,"d")
							n  = @cp_flange_database.get_flange_params(base,doc,du,"n")
							namedescript = @cp_flange_database.get_flange_params(base,doc,du,"Наименование_фланца")
							if (du!="") && (du!=nil) #addrow(du,dn,st,mas,name)
								js_command = "addrow(\"#{du}\",\"#{d1}\",\"#{d2}\",\"#{d3}\",\"#{d4}\",\"#{h1}\",\"#{h2}\",\"#{h3}\",\"#{d}\",\"#{n}\",\"#{namedescript}\")"
								dlg.execute_script(js_command) #запоняем список базы данных
							end
						end
					end
					#Устанавливаем текущие языковые параметры
					dlg.execute_script("document.getElementById('text_editlistflanges').innerHTML=\"#{$coolpipe_langDriver["Редактирование списка фланцев"]}\"")
					dlg.execute_script("document.getElementById('text_DiametrUslov').innerHTML=\"#{$coolpipe_langDriver["Диаметр условный"]}\"")
					dlg.execute_script("document.getElementById('text_NameForSpec').innerHTML=\"#{$coolpipe_langDriver["Наименование для спецификации"]}\"")
					dlg.execute_script("text_realdel=\"#{$coolpipe_langDriver["Действительно удалить:"]}\"")
					dlg.execute_script("document.getElementById('newdoc').value=\"#{$coolpipe_langDriver["Новый"]}\"")
					dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
					dlg.execute_script("document.getElementById('save').value=\"#{$coolpipe_langDriver["Сохранить"]}\"")
					dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
				when "cancel"
					dlg.close
				when "changelistflangeselements"
					#Редактируем список с базами
					@cp_change_list_flanges_dialog.close
					if arr[1]!="0"
						listdu=[]
						for i in 1..arr.length-1 
							listdu<<encode_to_utf8(arr[i])
						end
						@cp_flange_database.add_flanges(base,doc,listdu)
						cp_refreshdialog(@cp_activ_selectflange_dialog,"selectflange_dialog")
					end
				when "delete"
					@cp_flange_database.delete_tube(base,doc,encode_to_utf8(arr[1]))
					cp_refreshdialog(@cp_activ_selectflange_dialog,"selectflange_dialog")
			end
		end
		##################################################################################################
		#------- Спецификация
		##################################################################################################
		def cp_isconector1_2_1?(face)                         #возвращает TRUE если Face - является коннектором CoolPipe
			##########################################
			# Для совместимости версии 1.2.1
			##########################################
			rez = false
			if face.class==Sketchup::Face
				rez = true if face.get_attribute("CoolPipeComponent","Тип")=="Коннектор" #объект CoolPipe - Коннектор
			end
			rez
		end
		def cp_get_connectors_arr1_2_1(component)           #Получает все коннекторы объекта, если их нет - то возвращает nil
			##########################################
			# Для совместимости версии 1.2.1
			##########################################
			arr = []
			if (component!=nil) and (component.definition.entities[0].class==Sketchup::Group)
				component.definition.entities[0].entities.each {|ent|
					if cp_isconector1_2_1?(ent)
						arr = [] if arr==nil
						arr << ent
					end
				}
				if arr==nil
					component.definition.entities[0].entities.each {|ent|
						if ent.class==Sketchup::Group
							ent.entities.each {|ent1|
								if cp_isconector1_2_1?(ent1)
									arr = [] if arr==nil
									arr << ent1
								end
							}
						end
					}
				end
			end
			arr
		end
		def get_length_tube_1_2_1(tube,attributes)
			##########################################
			# Для совместимости версии 1.2.1
			##########################################
			length = 0
			connectors = cp_get_connectors_arr1_2_1(tube)
			if (connectors.length==2) && (attributes[:Тип]=="Труба")
				tr1 = tube.definition.entities[0].transformation #обратная трансформация группы для получения абсолютных координат
				tr2 = tube.transformation                        #обратная трансформация компонента для получения абсолютных координат
				pt1 = connectors[0].bounds.center                #получение абсолютной точки начала трубопровода
				pt1 = pt1.transform! tr1
				pt1 = pt1.transform! tr2
				pt2 = connectors[1].bounds.center                #получение абсолютной точки начала трубопровода
				pt2 = pt2.transform! tr1
				pt2 = pt2.transform! tr2
				vec = pt1 - pt2
				length = ((vec.length.to_m*100).round.to_f)/100
			end
			length
		end		
		def cp_get_length_tube(tube,attributes) #Метод определяет длину трубопровода
			length = 0
			connectors = Sketchup::CoolPipe::cp_get_connectors_arr(tube)
			if (connectors.length==2) && (attributes[:Тип]=="Труба")
				tr1 = tube.transformation                        #трансформация для получения текщих координат
				pt1 = connectors[0].position                     #получение текщей точки начала трубопровода
				pt1 = pt1.transform! tr1
				pt2 = connectors[1].position                     #получение текщей точки начала трубопровода
				pt2 = pt2.transform! tr1
				vec = pt1 - pt2
				length = ((vec.length.to_m*100).to_i.to_f)/100 if vec!=nil
				#puts "Участок трубы length = #{length} (#{attributes[:Имя]})"
			end
			if length==0 #Строится предположение что труба начерчена в версии 1.2.1
				length=get_length_tube_1_2_1(tube,attributes)
			end
			length 
		end
		def cp_get_index_from_arr(arr,el_spec)  #Возвращает индекс элемента спецификации если такой имеется, иначе результат -1
			rez = -1
			if arr!=nil
			if arr.length>0
				for i in 0..(arr.length-1)
					if (arr[i][0]==el_spec[0])&&(arr[i][1]==el_spec[1])&&(arr[i][3]==el_spec[3])#&&(arr[i][4]==el_spec[4])
						rez = i
						break
					end 
				end
			end
			end
			rez
		end
		def cp_addelement!(arr,attributes,type) #Добавляет элемент спецификации в массив arr
			if attributes[:Тип]==type
				element = [attributes[:Имя],attributes[:ЕдИзм],"1",attributes[:масса],attributes[:Dнар]]
				index = cp_get_index_from_arr(arr,element)
				if index>-1
					arr[index][2] = (arr[index][2].to_i+1).to_s
				else
					arr << element
				end 
			end
			#puts "arr1=#{arr}" if type=="Отвод"
			arr = arr.sort #Сортировка
			arr = arr.uniq #Уникальные элементы
			#puts "arr2=#{arr}" if type=="Отвод"
			arr
		end
		def cp_get_js_spec(arr)                 #создает js script для заполнения диалога "спецификация"
			js = ""
			arr = arr.uniq
			if arr!=nil
				if arr.length>0
					#### Выборка всех имен элементов и их сортировка по алфавиту
					sort_names = []
					arr.each {|el|;sort_names<<el[0];}
					sort_names = sort_names.sort
					####
					arr2 = [] #Сортированный массив элементов
					sort_names.each {|name|
						arr.each {|el|
						 arr2 << el if name==el[0]
						}
					}
					arr2=arr2.uniq
					####
					arr2.each {|el|
						js+="addrowspec(\"#{el[0]}\",\"#{el[1]}\",\"#{el[2]}\",\"#{el[3]}\");"
						@specification = @specification + "#{@numspec};#{el[0]};#{el[1]};#{el[2]};#{el[3]}\n"
						@numspec = @numspec+1
					}
				end
			end
			#puts "js = #{js}"
			js
		end
		def getSpecArray(entities)#РЕКУРСИВНАЯ ФУНКЦИЯ ищет все элементы CoolPipe для спецификации
			arr = []
			entities.each {|component|
				component_class = component.class.to_s
				is_cpComponent = Sketchup::CoolPipe::cp_iscpcomponent?(component)
				case component_class
					when "Sketchup::Group"
						if is_cpComponent
							arr << component 
						else
							arr = arr + getSpecArray(component.entities)
						end
					when "Sketchup::ComponentInstance"
						if is_cpComponent
							arr << component 
						else
							arr = arr + getSpecArray(component.definition.entities)
						end
				end
				arr.compact
			}
			arr
		end
		######
		def get_entities_for_specification #Проверка необходимости подсчета выделенной геометрии или считать все вместе
			entities = nil
			model = Sketchup.active_model
			selection = model.selection
			allentities  = model.entities
			puts "----------------------------"
			puts "#{$coolpipe_langDriver["Выделенных компонентов"]} = #{selection.length}" #Добавить в список перевода !!!!!!!!!!!!!!!!!
			puts "#{$coolpipe_langDriver["Всего объектов в модели"]} = #{allentities.length}" #Добавить в список перевода !!!!!!!!!!!!!!!!!
			if selection.length!=0
				message = "#{$coolpipe_langDriver["Посчитать только выделенную геометрию"]}?".to_sym #Добавить в список перевода !!!!!!!!!!!!!!!!!
				result = UI.messagebox(message, MB_YESNO)
			end
			if result == IDYES
				entities = selection
			else
				entities = allentities
			end
			entities
		end
		def generate_CoolPipe_specification			
			entities = get_entities_for_specification
			j = 0
			pipes =  [];elbows=  [];reducers=[]
			tees=    [];caps=    [];flanges= []
			oborudov=[];armatura=[];drugoe=[];count = 0
			any_langTypes=[$coolpipe_langDriver["Отвод"],    			 $coolpipe_langDriver["Переход"],
			               $coolpipe_langDriver["Тройник"],              $coolpipe_langDriver["Заглушка"], $coolpipe_langDriver["Фланец"],
					       $coolpipe_langDriver["Основное оборудование"],$coolpipe_langDriver["Арматура"], $coolpipe_langDriver["Другое"]];
			any_langElements=[[],[],[],[],[],[],[],[]]
			all_cpComponents = getSpecArray(entities) #Получение всех элементов Coolpipe (рекурсивная функция)
			#puts "all_cpComponents.length=#{all_cpComponents.length}"
			all_cpComponents.each {|component|
			#if Sketchup::CoolPipe::cp_iscpcomponent?(component)
				attributes = Sketchup::CoolPipe::cp_getattributes(component)
				if attributes[:Тип]=="Труба"
					length = cp_get_length_tube(component,attributes)
					attributes[:Длина]=length #Добавляем атрибут "Длина" - если component имеет тип "Труба"
					element = [attributes[:Имя],attributes[:ЕдИзм],length.to_s,attributes[:масса],attributes[:Dнар]]
					index = cp_get_index_from_arr(pipes,element)
					#puts "index = #{index}"
					if index>-1
						if (length!=nil)
							pipes[index][2] = pipes[index][2].to_f+length.to_f
							pipes[index][2] = (((pipes[index][2]*100).to_i).to_f/100).to_s
						end
					else
						pipes << element
					end
				end
				@specification = ""
				@numspec = 1
				cp_addelement!(elbows,attributes,"Отвод")
				cp_addelement!(reducers,attributes,"Переход")
				cp_addelement!(tees,attributes,"Тройник")
				cp_addelement!(caps,attributes,"Заглушка")
				cp_addelement!(flanges,attributes,"Фланец")
				cp_addelement!(oborudov,attributes,"Основное оборудование")
				cp_addelement!(armatura,attributes,"Арматура")
				cp_addelement!(drugoe,attributes,"Другое")
				if ("Труба")!=($coolpipe_langDriver["Труба"])
					for i in 0..(any_langTypes.length-1)
						cp_addelement!(any_langElements[i],attributes,any_langTypes[i]) 
					end		
				end
				count += 1
			#end
			}
			puts($coolpipe_langDriver["Количество переданных элементов в спецификацию="]+count.to_s)
			#вывод спецификации
			if count>0
				js_script = ""
				js_script+= cp_get_js_spec(oborudov)
				js_script+= cp_get_js_spec(armatura)
				js_script+= cp_get_js_spec(pipes)
				js_script+= cp_get_js_spec(elbows)
				js_script+= cp_get_js_spec(reducers)
				js_script+= cp_get_js_spec(tees)
				js_script+= cp_get_js_spec(caps)
				js_script+= cp_get_js_spec(flanges)
				js_script+= cp_get_js_spec(drugoe)
				if ("Труба")!=($coolpipe_langDriver["Труба"])
					for i in 0..(any_langTypes.length-1)
						js = cp_get_js_spec(any_langElements[i])
						js_script+= js if (js!=nil) and (js!="")
					end		
				end
				if @cp_specification_dialog == nil
					plugins=Sketchup.find_support_file("Plugins")
					@cp_specification_dialog=UI::WebDialog.new($coolpipe_langDriver["Спецификация элементов"], true, $coolpipe_langDriver["Спецификация элементов"],600,400,0,0,true) #width,height,left,top
					@cp_specification_dialog.set_file(File.join(Sketchup.find_support_file("Plugins/CoolPipe/html/specification.html")))
					color = @cp_specification_dialog.get_default_dialog_color 
					@cp_specification_dialog.set_background_color(color)
					@cp_specification_dialog.add_action_callback("ValueChanged") {|d,p|
					arr=p.split("|")
					dlg = @cp_specification_dialog
					case arr[0]
						when "spec_load_succesfull" #если окно загруженно заполняем его
							#Устанавливаем текущие языковые параметры
							dlg.execute_script("document.getElementById('text_Number').innerHTML=\"#{$coolpipe_langDriver["№"]}\"")
							dlg.execute_script("document.getElementById('text_Name').innerHTML=\"#{$coolpipe_langDriver["Наименование и характеристика"]}\"")
							dlg.execute_script("document.getElementById('text_EdIzm').innerHTML=\"#{$coolpipe_langDriver["Единица измерения"]}\"")
							dlg.execute_script("document.getElementById('text_Kolvo').innerHTML=\"#{$coolpipe_langDriver["Количество"]}\"")
							dlg.execute_script("document.getElementById('text_Massa').innerHTML=\"#{$coolpipe_langDriver["Масса единицы"]}\"")
							dlg.execute_script("document.getElementById('cancel').value=\"#{$coolpipe_langDriver["Отмена"]}\"")
							dlg.execute_script("document.getElementById('export_csv').value=\"#{$coolpipe_langDriver["Экспорт CSV"]}\"")
							dlg.execute_script("document.getElementById('text_copyright').innerHTML=\"#{$textcopyright}\"")
							dlg.execute_script(js_script) #запоняем спецификацию
						when "cancel"
							dlg.close
						when "exportCSV"
							text = "#{$coolpipe_langDriver["№"]};#{$coolpipe_langDriver["Наименование и характеристика"]};#{$coolpipe_langDriver["Единица измерения"]};#{$coolpipe_langDriver["Количество"]};#{$coolpipe_langDriver["Масса единицы"]}\n"
							text = text + @specification
							begin
								Dir.mkdir("c:/coolpipe/")
							rescue 
								puts "Папка \"c:/coolpipe/\" существует"
							end
							File.open("c:/coolpipe/specification.csv", "w") do |file|
								file.puts text
							end
							message = "#{$coolpipe_langDriver["Экспорт выполнен в файл:"]}\"c:/coolpipe/\""
							puts message
							UI.messagebox(message,MB_OK)
					end
					}
					@cp_specification_dialog.set_on_close{@cp_specification_dialog=nil} #при закрытии, уничтожаем ссылку на активный диалог
					@cp_specification_dialog.max_height = 700
					@cp_specification_dialog.max_width  = 700
					@cp_specification_dialog.min_height = 200
					@cp_specification_dialog.min_width  = 550
					@cp_specification_dialog.show 
				end
			else
				UI.messagebox($coolpipe_langDriver["Объекты CoolPipe не найдены"])
			end
		end # generate_CoolPipe_specification
		#############################################################################################
		#############################################################################################
		def encode_to_utf8(string)        #Декодирование символов от Sahi /* http://stroyka.in/ */
			# Автор:    sahi
			# Сайт:     http://stroyka.in/
			# Дата:     3.03.2013
			# Описание ->> скрипт кодирует текст в utf-8 команда -> encode_to_utf8(string) для 
			#              веб диалогов, результат от вебдиалога возвращается в кодировке ASCII-8BIT 
			#              для русского языка (взамен библиотеки iconv)
			######################
			encode_utf8 = {
			   0x80 => "\xD0\x82", 0x81 => "\xD0\x83",     0x82 => "\xE2\x80\x9A", 
			   0x83 => "\xD1\x93", 0x84 => "\xE2\x80\x9E", 0x85 => "\xE2\x80\xA6", 
			   0x86 => "\xE2\x80\xA0", 0x87 => "\xE2\x80\xA1", 0x88 => "\xE2\x82\xAC", 
			   0x89 => "\xE2\x80\xB0", 0x8A => "\xD0\x89",     0x8B => "\xE2\x80\xB9", 
			   0x8C => "\xD0\x8A", 0x8D => "\xD0\x8C", 0x8E => "\xD0\x8B", 
			   0x8F => "\xD0\x8F", 0x90 => "\xD1\x92", 0x91 => "\xE2\x80\x98", 
			   0x92 => "\xE2\x80\x99", 0x93 => "\xE2\x80\x9C", 0x94 => "\xE2\x80\x9D", 
			   0x95 => "\xE2\x80\xA2", 0x96 => "\xE2\x80\x93", 0x97 => "\xE2\x80\x94", 
			   0x99 => "\xE2\x84\xA2", 0x9A => "\xD1\x99",     0x9B => "\xE2\x80\xBA", 
			   0x9C => "\xD1\x9A", 0x9D => "\xD1\x9C", 0x9E => "\xD1\x9B", 
			   0x9F => "\xD1\x9F", 0xA0 => "\xC2\xA0", 0xA1 => "\xD0\x8E", 
			   0xA2 => "\xD1\x9E", 0xA3 => "\xD0\x88", 0xA4 => "\xC2\xA4", 
			   0xA5 => "\xD2\x90", 0xA6 => "\xC2\xA6", 0xA7 => "\xC2\xA7", 
			   0xA8 => "\xD0\x81", 0xA9 => "\xC2\xA9", 0xAA => "\xD0\x84", 
			   0xAB => "\xC2\xAB", 0xAC => "\xC2\xAC", 0xAD => "\xC2\xAD", 
			   0xAE => "\xC2\xAE", 0xAF => "\xD0\x87", 0xB0 => "\xC2\xB0", 
			   0xB1 => "\xC2\xB1", 0xB2 => "\xD0\x86", 0xB3 => "\xD1\x96", 
			   0xB4 => "\xD2\x91", 0xB5 => "\xC2\xB5", 0xB6 => "\xC2\xB6", 
			   0xB7 => "\xC2\xB7", 0xB8 => "\xD1\x91", 0xB9 => "\xE2\x84\x96", 
			   0xBA => "\xD1\x94", 0xBB => "\xC2\xBB", 0xBC => "\xD1\x98", 
			   0xBD => "\xD0\x85", 0xBE => "\xD1\x95", 0xBF => "\xD1\x97", 
			   0xC0 => "\xD0\x90", 0xC1 => "\xD0\x91", 0xC2 => "\xD0\x92", 
			   0xC3 => "\xD0\x93", 0xC4 => "\xD0\x94", 0xC5 => "\xD0\x95", 
			   0xC6 => "\xD0\x96", 0xC7 => "\xD0\x97", 0xC8 => "\xD0\x98", 
			   0xC9 => "\xD0\x99", 0xCA => "\xD0\x9A", 0xCB => "\xD0\x9B", 
			   0xCC => "\xD0\x9C", 0xCD => "\xD0\x9D", 0xCE => "\xD0\x9E", 
			   0xCF => "\xD0\x9F", 0xD0 => "\xD0\xA0", 0xD1 => "\xD0\xA1", 
			   0xD2 => "\xD0\xA2", 0xD3 => "\xD0\xA3", 0xD4 => "\xD0\xA4", 
			   0xD5 => "\xD0\xA5", 0xD6 => "\xD0\xA6", 0xD7 => "\xD0\xA7", 
			   0xD8 => "\xD0\xA8", 0xD9 => "\xD0\xA9", 0xDA => "\xD0\xAA", 
			   0xDB => "\xD0\xAB", 0xDC => "\xD0\xAC", 0xDD => "\xD0\xAD", 
			   0xDE => "\xD0\xAE", 0xDF => "\xD0\xAF", 0xE0 => "\xD0\xB0", 
			   0xE1 => "\xD0\xB1", 0xE2 => "\xD0\xB2", 0xE3 => "\xD0\xB3", 
			   0xE4 => "\xD0\xB4", 0xE5 => "\xD0\xB5", 0xE6 => "\xD0\xB6", 
			   0xE7 => "\xD0\xB7", 0xE8 => "\xD0\xB8", 0xE9 => "\xD0\xB9", 
			   0xEA => "\xD0\xBA", 0xEB => "\xD0\xBB", 0xEC => "\xD0\xBC", 
			   0xED => "\xD0\xBD", 0xEE => "\xD0\xBE", 0xEF => "\xD0\xBF", 
			   0xF0 => "\xD1\x80", 0xF1 => "\xD1\x81", 0xF2 => "\xD1\x82", 
			   0xF3 => "\xD1\x83", 0xF4 => "\xD1\x84", 0xF5 => "\xD1\x85", 
			   0xF6 => "\xD1\x86", 0xF7 => "\xD1\x87", 0xF8 => "\xD1\x88", 
			   0xF9 => "\xD1\x89", 0xFA => "\xD1\x8A", 0xFB => "\xD1\x8B", 
			   0xFC => "\xD1\x8C",0xFD => "\xD1\x8D", 0xFE => "\xD1\x8E",
			   0xFF => "\xD1\x8F" } 
			begin
				string = string.unpack("U*").pack("U*")
			rescue 	Exception => e
					nents = []
					if string!=nil #Дополнительное условие (а то в этом месте частенько вылетает)
						string.each_byte{|stg| 
						if(encode_utf8[stg])
							nents.push encode_utf8[stg]
						else
							nents.push stg.chr
						end
						} 
						string = nents.to_s
					end
			 end
			return string
		end #def encode_to_utf8(string)
	end #class CoolPipeDialogs
end #module Sketchup::CoolPipe